/*	Autor: David Malet
	Data: 14-05-2012
	Projecte: Sistema de Control Energtic
	Revisi: 3
	Base de Dades: Oracle 10g
*/


/* Funcions */


/* Funci que retorna la poblaci d'un contador */
create or replace FUNCTION fn_poblacio_contador(contador VARCHAR) RETURN VARCHAR IS
 poble VARCHAR(75);

BEGIN
   SELECT poblacio INTO poble FROM CONTADORS WHERE codi_contracte = contador;
   RETURN poble;
END;
/




/* Funci que calcula la porci de la potncia contractada per un contador que 
   correspon a cada centraleta de distribuci.
   s la potncia contractada pel nombre de centraletes de les que penja. */
create or replace FUNCTION fn_frac_pot_contador (contador VARCHAR) RETURN NUMBER IS

 fraccio NUMBER;
 centraletes INTEGER;

BEGIN

 SELECT COUNT(*) INTO centraletes FROM CENTRALETES_DE_CONTADORS
  WHERE codi_contracte = contador;

 SELECT potencia_contractada/centraletes INTO fraccio FROM CONTADORS
  WHERE codi_contracte = contador;

 RETURN fraccio;

END;
/





/* Funci que calcula la potncia contractada pels contadors que penjen d'una centraleta de distribuci.
   s a dir, el sumatori de les potncies contractades (fraccions) per cadascn dels contadors. */
create or replace FUNCTION fn_pot_contr_CD (centraleta VARCHAR) RETURN NUMBER IS

 potencia NUMBER;
 trobats INTEGER;

BEGIN

 -- Si no hi ha contadors que penjin de la centraleta, torno 0.
  SELECT COUNT(*) INTO trobats FROM CONTADORS c WHERE EXISTS
  (SELECT * FROM CENTRALETES_DE_CONTADORS r WHERE r.codi_central = centraleta AND
  r.codi_contracte = c.codi_contracte);
 
 IF trobats = 0 THEN
    potencia := 0.0;
 ELSE
    SELECT SUM(fn_frac_pot_contador(c.codi_contracte)) INTO potencia FROM CONTADORS c
     WHERE c.baixa = 0 AND EXISTS
     (SELECT * FROM CENTRALETES_DE_CONTADORS r WHERE r.codi_central = centraleta AND
     r.codi_contracte = c.codi_contracte);
 END IF;

 RETURN potencia;

END;
/



/* Funci que calcula la porci del sumatori de potncies contractades pels contadors que 
   penjen d'una centraleta de distribuci, que correspon a cada central de producci.
   s la potncia de la centraleta pel nombre de centrals  de producci de les que penja. */
create or replace FUNCTION fn_frac_pot_centraleta (centraleta VARCHAR) RETURN NUMBER IS

 fraccio NUMBER;
 centrals INTEGER;

BEGIN

 -- Considero la possibilitat que hi hagi redundncia de lnies
 -- A nivell de potncia, es contar com una sola lnia
 SELECT DISTINCT COUNT(codi_central_distribucio) INTO centrals FROM LINIES_DE_COMUNICACIO
  WHERE codi_central_distribucio = centraleta;

 fraccio := fn_pot_contr_CD(centraleta)/centrals;

 RETURN fraccio;

END;
/





/* Funci que calcula la potncia contractada pels contadors que penjen d'una central de producci.
   s a dir, el sumatori de les potncies contractades (fraccions) per cadascn dels contadors
   de cadascuna de les centraletes de distribucio. */
create or replace FUNCTION fn_pot_contr_CP (central VARCHAR) RETURN NUMBER IS

 potencia NUMBER;
 trobats INTEGER;

BEGIN

 -- Si no hi ha centraletes que penjin de la central, torno 0.
 SELECT DISTINCT COUNT(codi_central_distribucio) INTO trobats FROM LINIES_DE_COMUNICACIO
  WHERE codi_central_produccio = central;  

 IF trobats = 0 THEN
    potencia := 0.0;
 ELSE
     -- Considero la possibilitat que hi hagi redundncia de lnies
     -- A nivell de potncia, es contar com una sola lnia
    SELECT SUM(fn_frac_pot_centraleta(codi_central_distribucio)) INTO potencia FROM
     (SELECT DISTINCT codi_central_distribucio FROM LINIES_DE_COMUNICACIO
     WHERE codi_central_produccio = central);

 END IF;

 RETURN potencia;

END;
/



/* Funci de suport al procediment pr_contadors_mes_80pcnt_mitja, que retorna, per un contador
   i una data el consum del contador pel mes de la data.
   Si no hi hagus lectura anterior, es prendr 0 com a lectura inicial. */

create or replace FUNCTION fn_consum_mes(contador VARCHAR, data_interes DATE) RETURN NUMBER IS
 consum NUMBER;
 inicial NUMBER;
 trobats INTEGER;

BEGIN
   SELECT MAX(valor_lectura) INTO trobats FROM LECTURES
    WHERE data_lectura < data_interes AND codi_contracte = contador;

   IF trobats = 0 THEN
      inicial := 0.0;
   ELSE
      SELECT MAX(valor_lectura) INTO inicial FROM LECTURES
        WHERE data_lectura < data_interes AND codi_contracte = contador;
   END IF;

   SELECT valor_lectura - inicial INTO consum FROM LECTURES
    WHERE codi_contracte = contador
    AND EXTRACT (MONTH FROM data_lectura) = EXTRACT (MONTH FROM data_interes)
    AND EXTRACT (YEAR FROM data_lectura) = EXTRACT (YEAR FROM data_interes);

   RETURN consum;

END;
/



/* Funci de suport al procediment act_est_energia_linia, que retorna, per un contador i una data
   el consum del contador per l'any de la data.
   Si no hi hagus lectura de l'any anterior, es prendr 0 com a lectura inicial. */

create or replace FUNCTION fn_consum_contador(contador VARCHAR, data_interes DATE) RETURN NUMBER IS
 consum NUMBER;
 inicial NUMBER;
 trobats INTEGER;

BEGIN
   SELECT COUNT(valor_lectura) INTO trobats FROM LECTURES
    WHERE EXTRACT (YEAR FROM data_lectura) < EXTRACT (YEAR FROM data_interes)
    AND codi_contracte = contador;

   IF trobats = 0 THEN
      inicial := 0.0;
   ELSE
      SELECT MAX(valor_lectura) INTO inicial FROM LECTURES
        WHERE EXTRACT (YEAR FROM data_lectura) < EXTRACT (YEAR FROM data_interes)
        AND codi_contracte = contador;
   END IF;

   SELECT MAX(valor_lectura) - inicial INTO consum FROM LECTURES
    WHERE codi_contracte = contador
    AND EXTRACT (YEAR FROM data_lectura) = EXTRACT (YEAR FROM data_interes);

   RETURN consum;

END;
/




/* Funci de suport al procediment act_est_centrals_menys_30pcnt, que retorna,
   per una central de producci la seva capacitat mensual mxima. */

create or replace FUNCTION fn_capacitat_CP(central VARCHAR) RETURN NUMBER IS
 capacitat NUMBER;
 nom_de_taula VARCHAR(25);
 sql_string VARCHAR(200);

BEGIN
 SELECT nom_taula INTO nom_de_taula FROM TAULES_DE_CENTRALS
  WHERE codi_central = central;

 sql_string := 'SELECT capacitat_maxima FROM ' || nom_de_taula ||
  ' WHERE codi_central = :a';
 EXECUTE IMMEDIATE sql_string INTO capacitat USING central;

 RETURN capacitat;

END;
/
