/*	Autor: David Malet
	Data: 17-04-2012
	Projecte: Sistema de Control Energtic
	Revisi: 8
	Base de Dades: Oracle 10g
*/

/* Procediments d'Alta, Baixa i Modificaci de registres de les taules */


/* Procediments d'Alta */

/* Alta d'una Persona */
CREATE OR REPLACE PROCEDURE pr_alta_persona (alta_NIF VARCHAR, nom VARCHAR, cognoms VARCHAR,
 adrea VARCHAR, codi_postal NUMBER, poblacio VARCHAR, provincia VARCHAR,
 pais VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_NIF_nul EXCEPTION;
 exc_NIF_existeix EXCEPTION;
 exc_nom_nul EXCEPTION;
 exc_cognoms_nul EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_persona';
 parametres_entrada := 'NIF = ' || alta_NIF || '; nom = ' || nom || '; cognoms = ' || cognoms ||
    '; adrea = ' || adrea || '; codi_postal = ' || TO_CHAR(codi_postal) ||
    '; poblacio = ' || poblacio || '; provincia = ' || provincia || '; pais = ' || pais || ';';

 -- Excepci exc_NIF_nul, que es dona si no s'ha indicat el valor del NIF
 IF alta_NIF IS NULL THEN
    RAISE exc_NIF_nul;
 END IF;

 -- Excepci exc_NIF_existeix, que es dona si la persona (NIF) ja existeix
 SELECT COUNT(*) INTO trobats FROM PERSONES WHERE NIF = alta_NIF;
 IF trobats > 0 THEN
    RAISE exc_NIF_existeix;
 END IF;
  
 -- Excepci exc_nom_nul, que es dona si no s'ha indicat el valor del nom
 IF nom IS NULL THEN
    RAISE exc_nom_nul;
 END IF;

 -- Excepci exc_cognoms_nul, que es dona si no s'ha indicat el valor dels cognoms
 IF cognoms IS NULL THEN
    RAISE exc_cognoms_nul;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat el valor de l'adrea
 IF adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el valor del codi postal
 IF codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat el valor de la poblaci
 IF poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat el valor de la provncia
 IF provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el valor del pas
 IF pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;  

 -- Inserci de la persona en la taula PERSONES
 INSERT INTO PERSONES VALUES (alta_NIF, nom, cognoms, adrea, codi_postal,
    poblacio, provincia, pais);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_NIF_nul THEN 
    resultat := 'ERROR. Cal introduir el NIF de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_NIF_existeix THEN 
    resultat := 'ERROR. Ja existeix una persona amb el NIF ' || alta_NIF || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_nom_nul THEN 
    resultat := 'ERROR. Cal introduir el nom de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_cognoms_nul THEN 
    resultat := 'ERROR. Cal introduir els cognoms de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_adrea_nul THEN 
    resultat := 'ERROR. Cal introduir l''adrea de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_postal_nul THEN 
    resultat := 'ERROR. Cal introduir el Codi Postal de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_poblacio_nul THEN 
    resultat := 'ERROR. Cal introduir la poblaci de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_provincia_nul THEN 
    resultat := 'ERROR. Cal introduir la provncia de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_pais_nul THEN 
    resultat := 'ERROR. Cal introduir el pas de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT; 
END;
/


/* Alta d'un Contador */
CREATE OR REPLACE PROCEDURE pr_alta_contador (alta_codi_contracte VARCHAR, potencia VARCHAR,
 adrea VARCHAR, codi_postal NUMBER, poblacio VARCHAR, provincia VARCHAR, pais VARCHAR,
 marca VARCHAR, model_c VARCHAR, any_fabricacio NUMBER, numero_serie VARCHAR, client VARCHAR,
 resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_contracte_nul EXCEPTION;
 exc_contracte_existeix EXCEPTION;
 exc_potencia_nul EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
 exc_any_incorrecte EXCEPTION;
 exc_client_nul EXCEPTION;
 exc_client_no_existeix EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_contador';
 parametres_entrada := 'codi_contracte = ' || alta_codi_contracte || '; potencia = ' || potencia ||
 '; adrea = ' || adrea || '; codi_postal = ' || TO_CHAR(codi_postal) ||
 '; poblacio = ' || poblacio || '; provincia = ' || provincia || '; pais = ' || pais ||
 '; marca = ' || marca || '; model = ' || model_c || '; any_fabricacio = ' || TO_CHAR(any_fabricacio) ||
 '; numero_serie = ' || numero_serie || '; client = ' || client || ';';

 -- Excepci exc_contracte_nul, que es dona si no s'ha indicat el codi de contracte
 IF alta_codi_contracte IS NULL THEN
    RAISE exc_contracte_nul;
 END IF;

 -- Excepci exc_contracte_existeix, que es dona si el contracte (codi) ja existeix
 SELECT COUNT(*) INTO trobats FROM CONTADORS WHERE codi_contracte = alta_codi_contracte;
 IF trobats > 0 THEN
    RAISE exc_contracte_existeix;
 END IF;
  
 -- Excepci exc_potencia_nul, que es dona si no s'ha indicat el valor de la potncia
 IF potencia IS NULL THEN
    RAISE exc_potencia_nul;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea del contador
 IF adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal del contador
 IF codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci del contador
 IF poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia del contador
 IF provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas del contador
 IF pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF; 

 -- Excepci exc_any_incorrecte, que es dona si l'any especificat no es troba entre 1850 i 9999.
 IF any_fabricacio < 1850 OR any_fabricacio > 9999 OR TRUNC(any_fabricacio) <> any_fabricacio THEN
    RAISE exc_any_incorrecte;
 END IF;  

 -- Excepci exc_client_nul, que es dona si no s'ha indicat el NIF del client del contracte
 IF client IS NULL THEN
    RAISE exc_client_nul;
 END IF;

 -- Excepci exc_client_no_existeix, que es dona si el client (NIF) no existeix
 SELECT COUNT(*) INTO trobats FROM PERSONES WHERE NIF = client;
 IF trobats = 0 THEN
    RAISE exc_client_no_existeix;
 END IF;

 -- Inserci del contador en la taula CONTADORS
 -- El valor inicial per defecte del parmetre alta_disponibilitat, ser 0
 -- Afegim un 0 al final per indicar que el CONTADOR no est de baixa (l'estem donant d'alta)
 INSERT INTO CONTADORS VALUES (alta_codi_contracte, potencia, adrea, codi_postal, poblacio,
  provincia, pais, 0.0, '', marca, model_c, any_fabricacio, numero_serie, 0, client, 0);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_contracte_nul THEN 
    resultat := 'ERROR. Cal indicar un codi de contracte.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contracte_existeix THEN 
    resultat := 'ERROR. Ja existeix un contador amb el codi de contracte ' ||
     alta_codi_contracte || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_potencia_nul THEN 
    resultat := 'ERROR. Cal introduir la potncia contractada.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_adrea_nul THEN 
    resultat := 'ERROR. Cal introduir l''adrea del contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_postal_nul THEN 
    resultat := 'ERROR. Cal introduir el Codi Postal del contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_poblacio_nul THEN 
    resultat := 'ERROR. Cal introduir la poblaci del contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_provincia_nul THEN 
    resultat := 'ERROR. Cal introduir la provncia del contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_pais_nul THEN 
    resultat := 'ERROR. Cal introduir el pas del contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_any_incorrecte THEN 
    resultat := 'ERROR. L''any de fabricaci especificat pel contador s incorrecte.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_client_nul THEN 
    resultat := 'ERROR. Cal indicar el NIF del client del contracte corresponent al contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_client_no_existeix THEN 
    resultat := 'ERROR. El NIF ' || client || ' no existeix.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una Central de Producci Nuclear */
CREATE OR REPLACE PROCEDURE pr_alta_central_nuclear (alta_codi_central VARCHAR, adrea VARCHAR,
 codi_postal NUMBER, poblacio VARCHAR, provincia VARCHAR, pais VARCHAR, data_inspeccio DATE,
 capacitat_maxima NUMBER, energia_minima NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_central_nul EXCEPTION;
 exc_central_existeix EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
 exc_capacitat_maxima_nul EXCEPTION;
 exc_energia_minima_nul EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_central_nuclear';
 parametres_entrada := 'codi_central = ' || alta_codi_central || '; adrea = ' || adrea ||
 '; codi_postal = ' || TO_CHAR(codi_postal) || '; poblacio = ' || poblacio ||
 '; provincia = ' || provincia || '; pais = ' || pais ||
 '; data_inspeccio = ' || TO_CHAR(data_inspeccio) ||
 '; capacitat_maxima = ' || TO_CHAR(capacitat_maxima) ||
 '; energia_minima = ' || TO_CHAR(energia_minima) || ';';

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la central
 IF alta_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_existeix, que es dona si la central (codi) ja existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = alta_codi_central;
 IF trobats > 0 THEN
    RAISE exc_central_existeix;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea de la central
 IF adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal de la central
 IF codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci de la central
 IF poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia de la central
 IF provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas de la central
 IF pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;  

 -- Excepci exc_capacitat_maxima_nul, que es dona si no s'ha indicat la capacitat mxima
 -- de la central
 IF capacitat_maxima IS NULL THEN
    RAISE exc_capacitat_maxima_nul;
 END IF;

 -- Excepci exc_energia_minima_nul, que es dona si no s'ha indicat l'energia mnima per al
 -- funcionament de la central
 IF energia_minima IS NULL THEN
    RAISE exc_energia_minima_nul;
 END IF;

 -- Inserci de la central en la taula TAULES_DE_CENTRALS
 INSERT INTO TAULES_DE_CENTRALS VALUES (alta_codi_central, 'CENTRALS_NUCLEARS');
 -- Inserci de la central en la taula CENTRALS_NUCLEARS
 INSERT INTO CENTRALS_NUCLEARS VALUES (alta_codi_central, adrea, codi_postal, poblacio,
  provincia, pais, data_inspeccio, capacitat_maxima, energia_minima);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal indicar un codi de central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_existeix THEN 
    resultat := 'ERROR. Ja existeix una central amb el codi de central ' ||
     alta_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_adrea_nul THEN 
    resultat := 'ERROR. Cal introduir l''adrea de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_postal_nul THEN 
    resultat := 'ERROR. Cal introduir el Codi Postal de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_poblacio_nul THEN 
    resultat := 'ERROR. Cal introduir la poblaci de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_provincia_nul THEN 
    resultat := 'ERROR. Cal introduir la provncia de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_pais_nul THEN 
    resultat := 'ERROR. Cal introduir el pas de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_capacitat_maxima_nul THEN 
    resultat := 'ERROR. Cal indicar la capacitat mensual mxima de la central nuclear.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_energia_minima_nul THEN 
    resultat := 'ERROR. Cal indicar l''energia mnima per al funcionament de la central nuclear.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una Central de Producci Trmica */
CREATE OR REPLACE PROCEDURE pr_alta_central_termica (alta_codi_central VARCHAR, adrea VARCHAR,
 codi_postal NUMBER, poblacio VARCHAR, provincia VARCHAR, pais VARCHAR, data_inspeccio DATE,
 capacitat_maxima NUMBER, emissions_CO2 NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_central_nul EXCEPTION;
 exc_central_existeix EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_capacitat_maxima_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_central_termica';
 parametres_entrada := 'codi_central = ' || alta_codi_central || '; adrea = ' || adrea ||
 '; codi_postal = ' || TO_CHAR(codi_postal) || '; poblacio = ' || poblacio ||
 '; provincia = ' || provincia || '; pais = ' || pais ||
 '; data_inspeccio = ' || TO_CHAR(data_inspeccio) ||
 '; capacitat_maxima = ' || TO_CHAR(capacitat_maxima) ||
 '; emissions_CO2 = ' || TO_CHAR(emissions_CO2) || ';';

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la central
 IF alta_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_existeix, que es dona si la central (codi) ja existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = alta_codi_central;
 IF trobats > 0 THEN
    RAISE exc_central_existeix;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea de la central
 IF adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal de la central
 IF codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci de la central
 IF poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia de la central
 IF provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas de la central
 IF pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;  

 -- Excepci exc_capacitat_maxima_nul, que es dona si no s'ha indicat la capacitat mxima de la central
 IF capacitat_maxima IS NULL THEN
    RAISE exc_capacitat_maxima_nul;
 END IF;  


 -- Inserci de la central en la taula TAULES_DE_CENTRALS
 INSERT INTO TAULES_DE_CENTRALS VALUES (alta_codi_central, 'CENTRALS_TERMIQUES');
 -- Inserci de la central en la taula CENTRALS_TERMIQUES
 INSERT INTO CENTRALS_TERMIQUES VALUES (alta_codi_central, adrea, codi_postal, poblacio,
  provincia, pais, data_inspeccio, capacitat_maxima, emissions_CO2);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal indicar un codi de central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_existeix THEN 
    resultat := 'ERROR. Ja existeix una central amb el codi de central ' ||
     alta_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_adrea_nul THEN 
    resultat := 'ERROR. Cal introduir l''adrea de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_postal_nul THEN 
    resultat := 'ERROR. Cal introduir el Codi Postal de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_poblacio_nul THEN 
    resultat := 'ERROR. Cal introduir la poblaci de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_provincia_nul THEN 
    resultat := 'ERROR. Cal introduir la provncia de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_pais_nul THEN 
    resultat := 'ERROR. Cal introduir el pas de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_capacitat_maxima_nul THEN 
    resultat := 'ERROR. Cal introduir la capacitat mxima de la central trmica.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una Central de Producci de Carb */
CREATE OR REPLACE PROCEDURE pr_alta_central_carbo (alta_codi_central VARCHAR, adrea VARCHAR,
 codi_postal NUMBER, poblacio VARCHAR, provincia VARCHAR, pais VARCHAR, data_inspeccio DATE,
 capacitat_maxima NUMBER, emissions_CO2 NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_central_nul EXCEPTION;
 exc_central_existeix EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
 exc_capacitat_maxima_nul EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_central_carbo';
 parametres_entrada := 'codi_central = ' || alta_codi_central || '; adrea = ' || adrea ||
 '; codi_postal = ' || TO_CHAR(codi_postal) || '; poblacio = ' || poblacio ||
 '; provincia = ' || provincia || '; pais = ' || pais ||
 '; data_inspeccio = ' || TO_CHAR(data_inspeccio) ||
 '; capacitat_maxima = ' || TO_CHAR(capacitat_maxima) ||
 '; emissions_CO2 = ' || TO_CHAR(emissions_CO2) || ';';

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la central
 IF alta_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_existeix, que es dona si la central (codi) ja existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = alta_codi_central;
 IF trobats > 0 THEN
    RAISE exc_central_existeix;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea de la central
 IF adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal de la central
 IF codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci de la central
 IF poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia de la central
 IF provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas de la central
 IF pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;  

 -- Excepci exc_capacitat_maxima_nul, que es dona si no s'ha indicat la capacitat mxima de la central
 IF capacitat_maxima IS NULL THEN
    RAISE exc_capacitat_maxima_nul;
 END IF;  


 -- Inserci de la central en la taula TAULES_DE_CENTRALS
 INSERT INTO TAULES_DE_CENTRALS VALUES (alta_codi_central, 'CENTRALS_DE_CARBO');
 -- Inserci de la central en la taula CENTRALS_DE_CARBO
 INSERT INTO CENTRALS_DE_CARBO VALUES (alta_codi_central, adrea, codi_postal, poblacio,
  provincia, pais, data_inspeccio, capacitat_maxima, emissions_CO2);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal indicar un codi de central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_existeix THEN 
    resultat := 'ERROR. Ja existeix una central amb el codi de central ' ||
     alta_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_adrea_nul THEN 
    resultat := 'ERROR. Cal introduir l''adrea de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_postal_nul THEN 
    resultat := 'ERROR. Cal introduir el Codi Postal de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_poblacio_nul THEN 
    resultat := 'ERROR. Cal introduir la poblaci de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_provincia_nul THEN 
    resultat := 'ERROR. Cal introduir la provncia de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_pais_nul THEN 
    resultat := 'ERROR. Cal introduir el pas de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_capacitat_maxima_nul THEN 
    resultat := 'ERROR. Cal introduir la capacitat mxima de la central de carb.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una Central de Producci Elica */
CREATE OR REPLACE PROCEDURE pr_alta_central_eolica (alta_codi_central VARCHAR, adrea VARCHAR,
 codi_postal NUMBER, poblacio VARCHAR, provincia VARCHAR, pais VARCHAR, data_inspeccio DATE,
 capacitat_maxima NUMBER, numero_molins INTEGER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_central_nul EXCEPTION;
 exc_central_existeix EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
 exc_capacitat_maxima_nul EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_central_eolica';
 parametres_entrada := 'codi_central = ' || alta_codi_central || '; adrea = ' || adrea ||
 '; codi_postal = ' || TO_CHAR(codi_postal) || '; poblacio = ' || poblacio ||
 '; provincia = ' || provincia || '; pais = ' || pais ||
 '; data_inspeccio = ' || TO_CHAR(data_inspeccio) ||
 '; capacitat_maxima = ' || TO_CHAR(capacitat_maxima) ||
 '; numero_molins = ' || TO_CHAR(numero_molins) || ';';

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la central
 IF alta_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_existeix, que es dona si la central (codi) ja existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = alta_codi_central;
 IF trobats > 0 THEN
    RAISE exc_central_existeix;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea de la central
 IF adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal de la central
 IF codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci de la central
 IF poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia de la central
 IF provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas de la central
 IF pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;  

 -- Excepci exc_capacitat_maxima_nul, que es dona si no s'ha indicat la capacitat mxima de la central
 IF capacitat_maxima IS NULL THEN
    RAISE exc_capacitat_maxima_nul;
 END IF;  


 -- Inserci de la central en la taula TAULES_DE_CENTRALS
 INSERT INTO TAULES_DE_CENTRALS VALUES (alta_codi_central, 'CENTRALS_EOLIQUES');
 -- Inserci de la central en la taula CENTRALS_EOLIQUES
 INSERT INTO CENTRALS_EOLIQUES VALUES (alta_codi_central, adrea, codi_postal, poblacio,
  provincia, pais, data_inspeccio, capacitat_maxima, numero_molins);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal indicar un codi de central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_existeix THEN 
    resultat := 'ERROR. Ja existeix una central amb el codi de central ' ||
     alta_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_adrea_nul THEN 
    resultat := 'ERROR. Cal introduir l''adrea de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_postal_nul THEN 
    resultat := 'ERROR. Cal introduir el Codi Postal de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_poblacio_nul THEN 
    resultat := 'ERROR. Cal introduir la poblaci de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_provincia_nul THEN 
    resultat := 'ERROR. Cal introduir la provncia de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_pais_nul THEN 
    resultat := 'ERROR. Cal introduir el pas de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_capacitat_maxima_nul THEN 
    resultat := 'ERROR. Cal introduir la capacitat mxima de la central elica.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una Central de Producci Solar */
CREATE OR REPLACE PROCEDURE pr_alta_central_solar (alta_codi_central VARCHAR, adrea VARCHAR,
 codi_postal NUMBER, poblacio VARCHAR, provincia VARCHAR, pais VARCHAR, data_inspeccio DATE,
 capacitat_maxima NUMBER, numero_panells INTEGER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_central_nul EXCEPTION;
 exc_central_existeix EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
 exc_capacitat_maxima_nul EXCEPTION;
 exc_numero_panells_nul EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_central_solar';
 parametres_entrada := 'codi_central = ' || alta_codi_central || '; adrea = ' || adrea ||
 '; codi_postal = ' || TO_CHAR(codi_postal) || '; poblacio = ' || poblacio ||
 '; provincia = ' || provincia || '; pais = ' || pais ||
 '; data_inspeccio = ' || TO_CHAR(data_inspeccio) ||
 '; capacitat_maxima = ' || TO_CHAR(capacitat_maxima) ||
 '; numero_panells = ' || TO_CHAR(numero_panells) || ';';

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la central
 IF alta_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_existeix, que es dona si la central (codi) ja existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = alta_codi_central;
 IF trobats > 0 THEN
    RAISE exc_central_existeix;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea de la central
 IF adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal de la central
 IF codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci de la central
 IF poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia de la central
 IF provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas de la central
 IF pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;  

 -- Excepci exc_capacitat_maxima_nul, que es dona si no s'ha indicat la capacitat mxima de la central
 IF capacitat_maxima IS NULL THEN
    RAISE exc_capacitat_maxima_nul;
 END IF;  

 -- Excepci exc_numero_panells_nul, que es dona si no s'ha indicat el nombre de panells mnim
 -- per al funcionamient de la central
 IF numero_panells IS NULL THEN
    RAISE exc_numero_panells_nul;
 END IF; 


 -- Inserci de la central en la taula TAULES_DE_CENTRALS
 INSERT INTO TAULES_DE_CENTRALS VALUES (alta_codi_central, 'CENTRALS_SOLARS');
 -- Inserci de la central en la taula CENTRALS_SOLARS
 INSERT INTO CENTRALS_SOLARS VALUES (alta_codi_central, adrea, codi_postal, poblacio,
  provincia, pais, data_inspeccio, capacitat_maxima, numero_panells);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal indicar un codi de central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_existeix THEN 
    resultat := 'ERROR. Ja existeix una central amb el codi de central ' ||
     alta_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_adrea_nul THEN 
    resultat := 'ERROR. Cal introduir l''adrea de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_postal_nul THEN 
    resultat := 'ERROR. Cal introduir el Codi Postal de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_poblacio_nul THEN 
    resultat := 'ERROR. Cal introduir la poblaci de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_provincia_nul THEN 
    resultat := 'ERROR. Cal introduir la provncia de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_pais_nul THEN 
    resultat := 'ERROR. Cal introduir el pas de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_capacitat_maxima_nul THEN 
    resultat := 'ERROR. Cal introduir la capacitat mxima de la central solar.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_numero_panells_nul THEN 
    resultat := 'ERROR. Cal introduir el nombre mnim de panells per al funcionament de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una Centraleta de Distribuci */
CREATE OR REPLACE PROCEDURE pr_alta_central_distribucio (alta_codi_central VARCHAR, adrea VARCHAR,
 codi_postal NUMBER, poblacio VARCHAR, provincia VARCHAR, pais VARCHAR, data_inspeccio DATE,
 capacitat_maxima NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_central_nul EXCEPTION;
 exc_central_existeix EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
 exc_capacitat_maxima_nul EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_central_distribucio';
 parametres_entrada := 'codi_central = ' || alta_codi_central || '; adrea = ' || adrea ||
 '; codi_postal = ' || TO_CHAR(codi_postal) || '; poblacio = ' || poblacio ||
 '; provincia = ' || provincia || '; pais = ' || pais ||
 '; data_inspeccio = ' || TO_CHAR(data_inspeccio) ||
 '; capacitat_maxima = ' || TO_CHAR(capacitat_maxima) || ';';

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la central
 IF alta_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_existeix, que es dona si la central (codi) ja existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = alta_codi_central;
 IF trobats > 0 THEN
    RAISE exc_central_existeix;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea de la central
 IF adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal de la central
 IF codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci de la central
 IF poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia de la central
 IF provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas de la central
 IF pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;  

 -- Excepci exc_capacitat_maxima_nul, que es dona si no s'ha indicat la capacitat mxima de la central
 IF capacitat_maxima IS NULL THEN
    RAISE exc_capacitat_maxima_nul;
 END IF; 

 -- Inserci de la central en la taula TAULES_DE_CENTRALS
 INSERT INTO TAULES_DE_CENTRALS VALUES (alta_codi_central, 'CENTRALS_DE_DISTRIBUCIO');
 -- Inserci de la central en la taula CENTRALS_DE_DISTRIBUCIO
 INSERT INTO CENTRALS_DE_DISTRIBUCIO VALUES (alta_codi_central, adrea, codi_postal, poblacio,
  provincia, pais, data_inspeccio, capacitat_maxima);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal indicar un codi de central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_existeix THEN 
    resultat := 'ERROR. Ja existeix una central amb el codi de central ' ||
     alta_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_adrea_nul THEN 
    resultat := 'ERROR. Cal introduir l''adrea de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_postal_nul THEN 
    resultat := 'ERROR. Cal introduir el Codi Postal de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_poblacio_nul THEN 
    resultat := 'ERROR. Cal introduir la poblaci de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_provincia_nul THEN 
    resultat := 'ERROR. Cal introduir la provncia de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_pais_nul THEN 
    resultat := 'ERROR. Cal introduir el pas de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_capacitat_maxima_nul THEN 
    resultat := 'ERROR. Cal introduir la capacitat mxima de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una Lnia de Comunicaci */
CREATE OR REPLACE PROCEDURE pr_alta_linia (alta_codi_linia VARCHAR, central_produccio VARCHAR,
 central_distribucio VARCHAR, capacitat NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
 nom_de_taula VARCHAR(25);
 capacitat_Central NUMBER;
 suma_CDs NUMBER;
 sql_string VARCHAR(200);
 capacitat_CP NUMBER;
    
 -- Variables per excepcions 
 exc_codi_linia_nul EXCEPTION;
 exc_linia_existeix EXCEPTION;
 exc_central_produccio_nul EXCEPTION;
 exc_produccio_no_existeix EXCEPTION;
 exc_central_no_es_produccio EXCEPTION;
 exc_central_distribucio_nul EXCEPTION;
 exc_distribucio_no_existeix EXCEPTION;
 exc_central_no_es_distribucio EXCEPTION;
 exc_capacitat_nul EXCEPTION;
 exc_falta_capacitat_linia EXCEPTION;
 exc_falta_capacitat_CP EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_linia';
 parametres_entrada := 'codi_linia = ' || alta_codi_linia || '; central_produccio = ' || central_produccio ||
 '; central_distribucio = ' || central_distribucio || '; capacitat_maxima = ' || capacitat || ';';

 -- Excepci exc_codi_linia_nul, que es dona si no s'ha indicat el codi de la linia de comunicaci
 IF alta_codi_linia IS NULL THEN
    RAISE exc_codi_linia_nul;
 END IF;

 -- Excepci exc_linia_existeix, que es dona si la lnia de comunicaci (codi) ja existeix
 SELECT COUNT(*) INTO trobats FROM LINIES_DE_COMUNICACIO WHERE codi_linia = alta_codi_linia;
 IF trobats > 0 THEN
    RAISE exc_linia_existeix;
 END IF;

 -- Excepci exc_central_produccio_nul, que es dona si no s'ha indicat la central de producci de la linia
 IF central_produccio IS NULL THEN
    RAISE exc_central_produccio_nul;
 END IF;

 -- Excepci exc_produccio_no_existeix, que es dona si la central de producci indicada no existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = central_produccio;
 IF trobats = 0 THEN
    RAISE exc_produccio_no_existeix;
 END IF;

 -- Excepci exc_central_no_es_produccio, que es dona si el codi de la central indicat no correspon
 -- a una central de producci (s d'una centraleta de distribuci).
 SELECT nom_taula INTO nom_de_taula FROM TAULES_DE_CENTRALS WHERE codi_central = central_produccio;
 IF nom_de_taula = 'CENTRALS_DE_DISTRIBUCIO' THEN
    RAISE exc_central_no_es_produccio;
 END IF; 

 -- Excepci exc_central_distribucio_nul, que es dona si no s'ha indicat la central de distribuci de la linia
 IF central_distribucio IS NULL THEN
    RAISE exc_central_distribucio_nul;
 END IF;

 -- Excepci exc_distribucio_no_existeix, que es dona si la central de distribuci indicada no existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = central_distribucio;
 IF trobats = 0 THEN
    RAISE exc_distribucio_no_existeix;
 END IF;

 -- Excepci exc_central_no_es_distribucio, que es dona si el codi de la central indicat no correspon
 -- a una centraleta de distribuci.
 SELECT nom_taula INTO nom_de_taula FROM TAULES_DE_CENTRALS WHERE codi_central = central_distribucio;
 IF nom_de_taula <> 'CENTRALS_DE_DISTRIBUCIO' THEN
    RAISE exc_central_no_es_distribucio;
 END IF; 

 -- Excepci exc_capacitat_nul, que es dona si no s'ha indicat la capacitat mxima de la lnia
 IF capacitat IS NULL THEN
    RAISE exc_capacitat_nul;
 END IF;

 -- Excepci exc_falta_capacitat_linia, que es dona si la capacitat indicada per la lnia de comunicaci
 -- no es suficient per abastir la centraleta de distribuci.
 -- La capacitat de la centraleta de distribuci s mensual, i la de la lnia en KW. Aix, per verificar
 -- si la lnia t prou capacitat, considerar un mes de 30 dies, s a dir, 720 hores.
 SELECT capacitat_maxima INTO capacitat_Central FROM CENTRALS_DE_DISTRIBUCIO
  WHERE codi_central = central_distribucio;
 IF capacitat < capacitat_Central/720 THEN
    RAISE exc_falta_capacitat_linia;
 END IF;

 -- Excepci exc_falta_capacitat_CP, que es dona si la central de producci no t prou capacitat "lliure"
 -- per assumir l'associaci amb aquesta centraleta de distribuci.
 SELECT capacitat_maxima INTO capacitat_Central FROM CENTRALS_DE_DISTRIBUCIO
  WHERE codi_central = central_distribucio;
 SELECT nom_taula INTO nom_de_taula FROM TAULES_DE_CENTRALS WHERE codi_central = central_produccio;
 sql_string := 'SELECT capacitat_maxima FROM ' || nom_de_taula || ' WHERE codi_central = :a';
 EXECUTE IMMEDIATE sql_string INTO capacitat_CP USING central_produccio;
 SELECT SUM(d.capacitat_maxima) INTO suma_CDs FROM CENTRALS_DE_DISTRIBUCIO d
  WHERE d.codi_central = (SELECT DISTINCT l.codi_central_distribucio FROM LINIES_DE_COMUNICACIO l
  WHERE l.codi_central_produccio = central_produccio AND l.codi_central_distribucio = d.codi_central);
 IF capacitat_Central > capacitat_CP-suma_CDs THEN
    RAISE exc_falta_capacitat_CP;
 END IF;

 -- Inserci de la lnia en la taula LINIES_DE_COMUNICACIO
 INSERT INTO LINIES_DE_COMUNICACIO VALUES (alta_codi_linia, central_produccio, central_distribucio, capacitat);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_linia_nul THEN 
    resultat := 'ERROR. Cal indicar un codi de lnia de comunicaci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_linia_existeix THEN 
    resultat := 'ERROR. Ja existeix una lnia de comunicaci amb el codi de lnia ' ||
     alta_codi_linia || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_produccio_nul THEN 
    resultat := 'ERROR. Cal introduir la central de producci de la lnia de comunicaci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_produccio_no_existeix THEN 
    resultat := 'ERROR. La central ' || central_produccio || ' no existeix.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_no_es_produccio THEN 
    resultat := 'ERROR. El codi ' || central_produccio || ' no correspon a una central de producci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_distribucio_nul THEN 
    resultat := 'ERROR. Cal introduir la central de distribuci de la lnia de comunicaci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_distribucio_no_existeix THEN 
    resultat := 'ERROR. La centraleta ' || central_distribucio || ' no existeix.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_no_es_distribucio THEN 
    resultat := 'ERROR. El codi ' || central_produccio || ' no correspon a una centraleta de distribuci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_capacitat_nul THEN 
    resultat := 'ERROR. Cal introduir la capacitat mxima de la lnia de comunicaci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_falta_capacitat_linia THEN 
    resultat := 'ERROR. La lnia no t prou capacitat per assumir l''associaci
     amb la centraleta de distribuci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_falta_capacitat_CP THEN 
    resultat := 'ERROR. La Central de Producci no t prou capacitat per assumir l''associaci
     amb la centraleta de distribuci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una relaci Contador-Centraleta */
/* Abans de la inserci es comprobar la capacitat "lliure" de la centraleta, i
   si no s suficient, es produr una excepci */
CREATE OR REPLACE PROCEDURE pr_alta_centraleta_contador (alta_codi_contracte VARCHAR,
 alta_codi_central VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
 capacitat_total NUMBER;
 capacitat_ocupada NUMBER;
 capacitat_lliure NUMBER;
 potencia_contador NUMBER;
 estat_baixa NUMBER;
    
 -- Variables per excepcions 
 exc_codi_contracte_nul EXCEPTION;
 exc_contador_no_existeix EXCEPTION;
 exc_contador_baixa EXCEPTION;
 exc_codi_central_nul EXCEPTION;
 exc_central_no_existeix EXCEPTION;
 exc_contracte_central_existeix EXCEPTION;
 exc_falta_capacitat EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_centraleta_contador';
 parametres_entrada := 'codi_contracte = ' || alta_codi_contracte ||
 '; codi_central = ' || alta_codi_central || ';';

 -- Excepci exc_codi_contracte_nul, que es dona si no s'ha indicat el codi del contracte
 IF alta_codi_contracte IS NULL THEN
    RAISE exc_codi_contracte_nul;
 END IF;

 -- Excepci exc_contador_no_existeix, que es dona si el contador no existeix
 SELECT COUNT(*) INTO trobats FROM CONTADORS WHERE codi_contracte = alta_codi_contracte;
 IF trobats = 0 THEN
    RAISE exc_contador_no_existeix;
 END IF;

 -- Excepci exc_contador_baixa, que es dona si el contador est donat de baixa
 SELECT baixa INTO estat_baixa FROM CONTADORS WHERE codi_contracte = alta_codi_contracte;
 IF estat_baixa = 1 THEN
    RAISE exc_contador_baixa;
 END IF;

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la centraleta
 IF alta_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_no_existeix, que es dona si la centraleta de distribuci no existeix
 SELECT COUNT(*) INTO trobats FROM CENTRALS_DE_DISTRIBUCIO WHERE codi_central = alta_codi_central;
 IF trobats = 0 THEN
    RAISE exc_central_no_existeix;
 END IF;

 -- Excepci exc_contracte_central_existeix, que es dona si ja existeix una relaci entre el contador
 -- i la centraleta de distribuci indicats
 SELECT COUNT(*) INTO trobats FROM CENTRALETES_DE_CONTADORS WHERE (codi_central = alta_codi_central
  AND codi_contracte = alta_codi_contracte);
 IF trobats > 0 THEN
    RAISE exc_contracte_central_existeix;
 END IF;

 -- Excepci exc_falta_capacitat, que es dona si la centraleta no t prou capacitat lliure
 -- La capacitat de la centraleta de distribuci s mensual, i la del contracte en KW. Aix,
 -- per verificar la centraleta t prou capacitat, considerar un mes de 30 dies, s a dir, 720 hores.
 SELECT capacitat_maxima INTO capacitat_total FROM CENTRALS_DE_DISTRIBUCIO
  WHERE codi_central = alta_codi_central;
 capacitat_ocupada := 720*fn_pot_contr_CD(alta_codi_central);
 capacitat_lliure := capacitat_total - capacitat_ocupada;
 SELECT potencia_contractada INTO potencia_contador FROM CONTADORS
  WHERE codi_contracte = alta_codi_contracte;
 IF 720*potencia_contador > capacitat_lliure THEN
    RAISE exc_falta_capacitat;
 END IF;

 -- Inserci de la relaci en la taula CENTRALETES_DE_CONTADORS
 INSERT INTO CENTRALETES_DE_CONTADORS VALUES (alta_codi_contracte, alta_codi_central);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_contracte_nul THEN 
    resultat := 'ERROR. Cal indicar el codi del contracte corresponent al contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contador_no_existeix THEN 
    resultat := 'ERROR. El contador ' || alta_codi_contracte || ' no existeix.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contador_baixa THEN 
    resultat := 'ERROR. El contracte ' || alta_codi_contracte || ' est donat de baixa.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal indicar el codi de la centraleta de distribuci';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_no_existeix THEN 
    resultat := 'ERROR. La centraleta de distribuci ' || alta_codi_central || ' no existeix';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contracte_central_existeix THEN 
    resultat := 'ERROR. Ja existeix una relaci entre el contador ' || alta_codi_contracte ||
     ' i la centraleta ' || alta_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_falta_capacitat THEN 
    resultat := 'ERROR. La centraleta de distribuci no t prou capacitat lliure.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Alta d'una modalitat de lectura */
CREATE OR REPLACE PROCEDURE pr_alta_modalitat_lectura (alta_codi_modalitat INTEGER,
 alta_nom_modalitat VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_modalitat_nul EXCEPTION;
 exc_nom_modalitat_nul EXCEPTION;
 exc_codi_modalitat_existeix EXCEPTION;
 exc_nom_modalitat_existeix EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_modalitat_lectura';
 parametres_entrada := 'codi_modalitat = ' || alta_codi_modalitat ||
 '; nom_modalitat = ' || alta_nom_modalitat || ';';

 -- Excepci exc_codi_modalitat_nul, que es dona si no s'ha indicat el codi de la modalitat
 IF alta_codi_modalitat IS NULL THEN
    RAISE exc_codi_modalitat_nul;
 END IF;

 -- Excepci exc_nom_modalitat_nul, que es dona si no s'ha indicat el nom de la modalitat
 IF alta_nom_modalitat IS NULL THEN
    RAISE exc_nom_modalitat_nul;
 END IF;

 -- Excepci exc_codi_modalitat_existeix, que es dona si ja existeix el codi de modalitat indicat
 SELECT COUNT(*) INTO trobats FROM MODALITATS_LECTURA WHERE codi_modalitat = alta_codi_modalitat;
 IF trobats > 0 THEN
    RAISE exc_codi_modalitat_existeix;
 END IF;

 -- Excepci exc_nom_modalitat_existeix, que es dona si ja existeix la modalitat indicada
 SELECT COUNT(*) INTO trobats FROM MODALITATS_LECTURA WHERE nom_modalitat = alta_nom_modalitat;
 IF trobats > 0 THEN
    RAISE exc_nom_modalitat_existeix;
 END IF;

 -- Inserci de la relaci en la taula CENTRALETES_DE_CONTADORS
 INSERT INTO MODALITATS_LECTURA VALUES (alta_codi_modalitat, alta_nom_modalitat);
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_modalitat_nul THEN 
    resultat := 'ERROR. Cal indicar el codi de la modalitat de lectura.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_nom_modalitat_nul THEN 
    resultat := 'ERROR. Cal indicar el nom de la modalitat de lectura';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_modalitat_existeix THEN 
    resultat := 'ERROR. Ja existeix una modalitat de lectura amb el codi ' || alta_codi_modalitat || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_nom_modalitat_existeix THEN 
    resultat := 'ERROR. Ja existeix una modalitat de lectura amb el nom ' || alta_nom_modalitat || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/


/* Actualitzaci del consum mensual mig d'un contador */
/* Procediment cridat per pr_alta_lectura_contador */
CREATE OR REPLACE PROCEDURE pr_act_consum_mens_mig (act_contracte VARCHAR, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 consum_total NUMBER;
 num_lectures INTEGER;
 consum_mig NUMBER;

 -- Variables per excepcions 
 -- No cal verificar el codi de contracte, perqu s'ha verificat en pr_alta_lectura_contador
 -- Sempre hi haur almenys una lectura, donat que l'ha inserit el procediment que crida aquest.

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_act_consum_mens_mig';
 parametres_entrada := 'codi_contracte = ' || act_contracte || ';';

 -- Obtenci del nombre de lectures
 SELECT COUNT(*) INTO num_lectures FROM LECTURES WHERE codi_contracte = act_contracte;
 -- El consum total v donat pel valor de la darrera lectura
 SELECT valor_lectura INTO consum_total FROM LECTURES WHERE codi_contracte = act_contracte AND
  data_lectura = (SELECT MAX(data_lectura) FROM LECTURES WHERE codi_contracte = act_contracte);
 -- Clcul del consum mig
 consum_mig := consum_total/num_lectures;

 -- Actualitzaci del valor consum mensual en el contador
 UPDATE CONTADORS SET consum_mensual_mig = consum_mig WHERE codi_contracte = act_contracte;
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
END;
/



/* Actualitzaci de l'energia mensual produda per una o ms centrals de producci,
   en funci d'una nova lectura de contador */
/* Procediment cridat per pr_act_energia_emesa */
CREATE OR REPLACE PROCEDURE pr_act_energia_produida (act_centraleta VARCHAR, act_data DATE,
 act_valor_mes NUMBER, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 TYPE central IS RECORD(codi_central VARCHAR(50));
 TYPE centrals IS TABLE OF central;
 taula_temp centrals;
 nombre_centrals INTEGER;
 mes_lectura VARCHAR(9);
 any_lectura NUMBER(4);
 porcio NUMBER;
 trobats INTEGER;

 -- Variables per excepcions 
 -- No cal verificar el codi de contracte ni data ni valor, perqu s'han verificat en
 -- pr_alta_lectura_contador

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_act_energia_produida';
 parametres_entrada := 'codi_centraleta = ' || act_centraleta || '; data = ' || TO_CHAR(act_data) ||
  '; valor_energia_mes = ' || act_valor_mes || ';';

 mes_lectura := EXTRACT(MONTH FROM act_data);
 any_lectura := EXTRACT(YEAR FROM act_data);

 -- Obt les centrals de producci de les que "penja" la centraleta de distribuci i hi afegeix la porci
 -- del consum (valor) que correspon, pel mes i any de la lectura (data)
 SELECT COUNT(*) INTO nombre_centrals FROM LINIES_DE_COMUNICACIO
  WHERE codi_central_distribucio = act_centraleta;
 SELECT codi_central_produccio BULK COLLECT INTO taula_temp FROM LINIES_DE_COMUNICACIO
  WHERE codi_central_distribucio = act_centraleta;
 porcio := act_valor_mes/nombre_centrals;
 FOR i IN taula_temp.FIRST .. taula_temp.LAST LOOP
    -- Miro si hi ha un registre pel mes i l'any de la lectura
    -- Si no n'hi ha, l'haur de crear
    SELECT COUNT(*) INTO trobats FROM CP_HISTORIC_PRODUCCIO
     WHERE codi_central = taula_temp(i).codi_central
     AND EXTRACT(MONTH FROM data_historic) = mes_lectura
     AND EXTRACT(YEAR FROM data_historic) = any_lectura;
    IF trobats = 0 THEN
       INSERT INTO CP_HISTORIC_PRODUCCIO VALUES(taula_temp(i).codi_central, act_data, porcio);
    ELSE
       UPDATE CP_HISTORIC_PRODUCCIO SET energia_absoluta = energia_absoluta + porcio
	WHERE codi_central = taula_temp(i).codi_central
        AND EXTRACT(MONTH FROM data_historic) = mes_lectura
        AND EXTRACT(YEAR FROM data_historic) = any_lectura;
    END IF;
 END LOOP;
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
END;
/



/* Actualitzaci de l'energia mensual emesa per una o ms centraletes de distribuci,
   en funci d'una nova lectura de contador */
/* Procediment cridat per pr_alta_lectura_contador */
CREATE OR REPLACE PROCEDURE pr_act_energia_emesa (act_contracte VARCHAR, act_data DATE,
 act_valor NUMBER, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 TYPE centraleta IS RECORD(codi_centraleta VARCHAR(50));
 TYPE centraletes IS TABLE OF centraleta;
 taula_temp centraletes;
 nombre_centraletes INTEGER;
 mes_lectura VARCHAR(9);
 any_lectura NUMBER(4);
 porcio NUMBER;
 trobats INTEGER;

 -- Variables per excepcions 
 -- No cal verificar el codi de contracte ni data ni valor, perqu s'han verificat en
 -- pr_alta_lectura_contador

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_act_energia_emesa';
 parametres_entrada := 'codi_contracte = ' || act_contracte || '; data = ' || TO_CHAR(act_data) ||
  '; valor_energia = ' || act_valor || ';';

 mes_lectura := EXTRACT(MONTH FROM act_data);
 any_lectura := EXTRACT(YEAR FROM act_data);

 -- Obt les centraletes de les que "penja" el contador i hi afegeix la porci
 -- del consum (valor) que correspon, pel mes i any de la lectura (data)
 SELECT COUNT(*) INTO nombre_centraletes FROM CENTRALETES_DE_CONTADORS
  WHERE codi_contracte = act_contracte;
 SELECT codi_central BULK COLLECT INTO taula_temp FROM CENTRALETES_DE_CONTADORS
  WHERE codi_contracte = act_contracte;
 porcio := act_valor/nombre_centraletes;
 FOR i IN taula_temp.FIRST .. taula_temp.LAST LOOP
    -- Miro si hi ha un registre pel mes i l'any de la lectura
    -- Si no n'hi ha, l'haur de crear
    SELECT COUNT(*) INTO trobats FROM CD_HISTORIC_EMISSIO
     WHERE codi_central = taula_temp(i).codi_centraleta
     AND EXTRACT(MONTH FROM data_historic) = mes_lectura
     AND EXTRACT(YEAR FROM data_historic) = any_lectura;
    IF trobats = 0 THEN
       INSERT INTO CD_HISTORIC_EMISSIO VALUES(taula_temp(i).codi_centraleta, act_data,
        porcio);
    ELSE
       UPDATE CD_HISTORIC_EMISSIO SET energia_absoluta = energia_absoluta + porcio
	WHERE codi_central = taula_temp(i).codi_centraleta
        AND EXTRACT(MONTH FROM data_historic) = mes_lectura
        AND EXTRACT(YEAR FROM data_historic) = any_lectura;
    END IF;
    -- Crido procediment per actualitzar energia produda de les centrals de producci associades
    pr_act_energia_produida (taula_temp(i).codi_centraleta, act_data, porcio, resultat);
 END LOOP;
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
END;
/



/* Alta d'una lectura de contador */
/* Un cop feta la inserci en la taula de LECTURES,
   s'actualitzaran els valors del consum mensual mig del contador,
   l'energia mxima emesa per les centraletes de distribuci associades (si es supera el mxim actual)
   i l'energia absoluta mensual produda per les centrals de producci associades.
   Tamb s'actualitzar la data de la darrera lectura del contador.
   Tamb es faran les diferents crides per l'actualitzaci de les taules del Mdul Estadstic. */
CREATE OR REPLACE PROCEDURE pr_alta_lectura_contador (alta_codi_contracte VARCHAR, alta_data_lectura DATE,
 valor_lectura NUMBER, modalitat_lectura INTEGER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
 estat_baixa NUMBER;
 darrera_lectura DATE;
 lectura_previa NUMBER;
 data_previa DATE;
 valor_mes NUMBER;
    
 -- Variables per excepcions 
 exc_codi_contracte_nul EXCEPTION;
 exc_contracte_no_existeix EXCEPTION;
 exc_contracte_baixa EXCEPTION;
 exc_no_CD EXCEPTION;
 exc_no_CP EXCEPTION;
 exc_data_nul EXCEPTION;
 exc_valor_nul EXCEPTION;
 exc_contracte_data_existeix EXCEPTION;
 exc_error_crida_fill EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_alta_lectura_contador';
 parametres_entrada := 'codi_contracte = ' || alta_codi_contracte || '; data_lectura = ' || alta_data_lectura ||
 '; valor_lectura = ' || valor_lectura || '; modalitat_lectura = ' || modalitat_lectura || ';';

 -- Excepci exc_codi_contracte_nul, que es dona si no s'ha indicat el codi del contracte
 IF alta_codi_contracte IS NULL THEN
    RAISE exc_codi_contracte_nul;
 END IF;

 -- Excepci exc_contracte_no_existeix, que es dona si no el contracte no existeix
 SELECT COUNT(*) INTO trobats FROM CONTADORS WHERE codi_contracte = alta_codi_contracte;
 IF trobats = 0 THEN
    RAISE exc_contracte_no_existeix;
 END IF;

 -- Excepci exc_contracte_baixa, que es dona si el contador est donat de baixa
 SELECT baixa INTO estat_baixa FROM CONTADORS WHERE codi_contracte = alta_codi_contracte;
 IF estat_baixa = 1 THEN
    RAISE exc_contracte_baixa;
 END IF;

 -- Excepci exc_no_CD, que es dona si el contador no est associat a cap centraleta de distribuci
 -- Si no penja d'una centraleta de distribuci, no pot rebre subministre.
 SELECT COUNT(*) INTO trobats FROM CENTRALETES_DE_CONTADORS WHERE codi_contracte = alta_codi_contracte;
 IF trobats = 0 THEN
    RAISE exc_no_CD;
 END IF;

 -- Excepci exc_no_CP, que es dona si el contador no est associat a cap central de producci
 -- Si no penja d'alguna central de producci, no pot rebre subministre.
 SELECT COUNT(*) INTO trobats FROM LINIES_DE_COMUNICACIO
  WHERE codi_central_distribucio IN (SELECT DISTINCT codi_central FROM CENTRALETES_DE_CONTADORS
   WHERE codi_contracte = alta_codi_contracte);
 IF trobats = 0 THEN
    RAISE exc_no_CP;
 END IF;

 -- Excepci exc_data_nul, que es dona si no s'ha indicat la data de la lectura de contador
 IF alta_data_lectura IS NULL THEN
    RAISE exc_data_nul;
 END IF;

 -- Excepci exc_valor_nul, que es dona si no s'ha indicat el valor de la lectura de contador
 IF valor_lectura IS NULL THEN
    RAISE exc_valor_nul;
 END IF;

 -- Excepci exc_contracte_data_existeix, que es dona si ja existeix una lectura per la mateixa data
 -- i el mateix contador
 SELECT COUNT(*) INTO trobats FROM LECTURES WHERE (codi_contracte = alta_codi_contracte AND
  data_lectura = alta_data_lectura);
 IF trobats > 0 THEN
    RAISE exc_contracte_data_existeix;
 END IF;

 -- Obt el valor del consum del mes anterior
 -- Si no hi ha lectura anterior assumirem que s 0, s a dir, que ens trobem
 -- amb la primera lectura pel contador en qesti
 SELECT COUNT(*) INTO trobats FROM LECTURES WHERE codi_contracte = alta_codi_contracte;
 IF trobats = 0 THEN
    lectura_previa := 0.0;
 ELSE
    SELECT valor_lectura INTO lectura_previa FROM LECTURES
     WHERE codi_contracte = alta_codi_contracte AND data_lectura = (SELECT MAX(data_lectura)
     FROM LECTURES WHERE codi_contracte = alta_codi_contracte);
 END IF;

 -- Inserci de la lectura en la taula LECTURES
 INSERT INTO LECTURES VALUES (alta_codi_contracte, alta_data_lectura, valor_lectura, modalitat_lectura);
 resultat := 'OK';

 -- Actualitzaci de la data de la darrera lectura de contador
 SELECT data_darrera_lectura INTO darrera_lectura FROM CONTADORS
  WHERE codi_contracte = alta_codi_contracte;
 IF darrera_lectura IS NULL THEN
    UPDATE CONTADORS SET data_darrera_lectura = alta_data_lectura
     WHERE codi_contracte = alta_codi_contracte;
 ELSE
    IF alta_data_lectura > darrera_lectura THEN
       UPDATE CONTADORS SET data_darrera_lectura = alta_data_lectura
        WHERE codi_contracte = alta_codi_contracte;
    END IF;
 END IF;

 -- Crida al procediment pr_act_consum_mens_mig
 -- per actualitzar el valor del consum mig mensual del contador
 pr_act_consum_mens_mig(alta_codi_contracte, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Crida al procediment pr_act_energia_emesa
 -- per actualitzar el valor de l'energia mensual emesa per les centraletes de distribuci
 -- associades al contador
 valor_mes := valor_lectura-lectura_previa;
 pr_act_energia_emesa(alta_codi_contracte, alta_data_lectura, valor_mes, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Crida al procediment per l'actualitzaci de la taula EST_CONSUM_PER_CENTRAL,
 -- del Mdul Estadstic
 act_est_consum_central (alta_codi_contracte, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF; 

 -- Crida al procediment per l'actualitzaci de la taula EST_ENERGIA_MITJA_ANY_LINIA
 -- del Mdul Estadstic
 act_est_energia_linia (alta_codi_contracte, alta_data_lectura, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF; 

 -- Crida al procediment per l'actualitzaci de la taula EST_LINIA_MES_CARREGADA
 -- del Mdul Estadstic
 act_est_linia_mes_carregada (resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF; 

 -- Crida al procediment per l'actualitzaci de la taula EST_PERC_LINES_MES_50PCNT
 -- del Mdul Estadstic
 act_est_linies_mes_50pcnt (alta_data_lectura, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF; 

 -- Crida al procediment per l'actualitzaci de la taula EST_N_CENTRALS_MENYS_30PCNT
 -- del Mdul Estadstic
 act_est_centrals_menys_30pcnt (alta_data_lectura, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Crida al procediment per l'actualitzaci de la taula EST_10_CONTADORS_MAXIM_CONSUM
 -- del Mdul Estadstic
 act_est_top10_contadors (resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Crida al procediment per l'actualitzaci de la taula EST_CONSUM_MIG_PER_CLIENT
 -- del Mdul Estadstic
 act_est_consum_clients (alta_codi_contracte, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_contracte_nul THEN 
    resultat := 'ERROR. Cal indicar el codi de contracte del contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contracte_no_existeix THEN 
    resultat := 'ERROR. El codi de contracte del contador no existeix.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contracte_baixa THEN 
    resultat := 'ERROR. El contracte ' || alta_codi_contracte || ' est donat de baixa.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_no_CD THEN 
    resultat := 'ERROR. El contador no est associat a cap Centraleta de Distribuci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_no_CP THEN 
    resultat := 'ERROR. El contador no est associat a cap Central de Producci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_data_nul THEN 
    resultat := 'ERROR. Cal indicar la data de la lectura de contador';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_valor_nul THEN 
    resultat := 'ERROR. Cal indicar el valor de la lectura de contador';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contracte_data_existeix THEN 
    resultat := 'ERROR. Ja existeix una lectura per la data ' || alta_data_lectura ||
     ' i el contador amb codi de contracte ' || alta_codi_contracte || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
  WHEN exc_error_crida_fill THEN 
     resultat := 'ERROR. Error en la crida a un procediment niuat.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/









/* Procediments de Modificaci */

/* Modificaci d'una Persona */
 -- Noms se'n podr modificar l'adrea (adrea, codi postal, poblaci, provncia i pas)
CREATE OR REPLACE PROCEDURE pr_modificacio_adrea_persona (mod_NIF VARCHAR, mod_adrea VARCHAR,
 mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR, mod_pais VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_NIF_nul EXCEPTION;
 exc_NIF_no_existeix EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_modificacio_adrea_persona';
 parametres_entrada := 'NIF = ' || mod_NIF || '; adrea = ' || mod_adrea ||
  '; codi_postal = ' || mod_codi_postal || '; poblacio = ' || mod_poblacio ||
  '; provincia = ' || mod_provincia || '; pais = ' || mod_pais || ';';
  
 -- Excepci exc_NIF_nul, que es dona si no s'ha indicat el NIF de la persona
 IF mod_NIF IS NULL THEN
    RAISE exc_NIF_nul;
 END IF;

 -- Excepci exc_NIF_no_existeix, que es dona si la persona no existeix
 SELECT COUNT(*) INTO trobats FROM PERSONES WHERE NIF = mod_NIF;
 IF trobats = 0 THEN
    RAISE exc_NIF_no_existeix;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea de la persona
 IF mod_adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal de la persona
 IF mod_codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci de la persona
 IF mod_poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia de la persona
 IF mod_provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas de la persona
 IF mod_pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;

 -- Actualitzaci d'una persona en la taula PERSONES  
 UPDATE PERSONES SET adrea = mod_adrea, codi_postal = mod_codi_postal, poblacio = mod_poblacio,
  provincia = mod_provincia, pais = mod_pais WHERE NIF = mod_NIF;  
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_NIF_nul THEN 
     resultat := 'ERROR. Cal indicar el NIF de la persona.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_NIF_no_existeix THEN 
     resultat := 'ERROR. No existeix la persona amb NIF ' || mod_NIF || '.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_adrea_nul THEN 
     resultat := 'ERROR. Cal indicar l''adrea de la persona.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_codi_postal_nul THEN 
     resultat := 'ERROR. Cal indicar el codi postal de la persona.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_poblacio_nul THEN 
     resultat := 'ERROR. Cal indicar la poblaci de la persona.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_provincia_nul THEN 
     resultat := 'ERROR. Cal indicar la provncia de la persona.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_pais_nul THEN 
     resultat := 'ERROR. Cal indicar el pas de la persona.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'un Contador */
 -- Se'n podr modificar l'adrea (adrea, codi postal, poblaci, provncia i pas),
 -- la potncia contractada, marca, model, any de fabricaci i nmero de srie
 -- (renovaci de contador per un contracte) i el servei d'alta disponibilitat
 -- per donar-lo de baixa, hi ha el procediment corresponent (no es pot modificar baixa)
CREATE OR REPLACE PROCEDURE pr_modificacio_contador (mod_codi_contracte VARCHAR, mod_potencia NUMBER,
 mod_adrea VARCHAR, mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR,
 mod_pais VARCHAR, mod_marca VARCHAR, mod_model VARCHAR, mod_any_fabricacio NUMBER,
 mod_numero_serie VARCHAR, mod_alta_disp NUMBER, mod_client VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
 pot_vella NUMBER;
 TYPE registre IS RECORD(codi_central VARCHAR(50), capacitat NUMBER);
 TYPE registres IS TABLE OF registre;
 taula_temp registres;
 capacitat_ocupada NUMBER;
 capacitat_lliure NUMBER;
    
 -- Variables per excepcions 
 exc_codi_contracte_nul EXCEPTION;
 exc_contador_no_existeix EXCEPTION;
 exc_potencia_nul EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
 exc_any_incorrecte EXCEPTION;
 exc_alta_disp_nul EXCEPTION;
 exc_no_dues_CDs EXCEPTION;
 exc_falta_capacitat_CD EXCEPTION;
 exc_client_nul EXCEPTION;
 exc_client_no_existeix EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_modificacio_contador';
 parametres_entrada := 'codi_contracte = ' || mod_codi_contracte ||
  '; potencia_contractada = ' || mod_potencia || '; adrea = ' || mod_adrea ||
  '; codi_postal = ' || mod_codi_postal || '; poblacio = ' || mod_poblacio ||
  '; provincia = ' || mod_provincia || '; pais = ' || mod_pais ||
  '; marca = ' || mod_marca || '; model = ' || mod_model ||
  '; any_fabricacio = ' || mod_any_fabricacio || '; numero_serie = ' || mod_numero_serie ||
  '; servei_alta_disponibilitat = ' || mod_alta_disp || ';';
  
 -- Excepci exc_codi_contracte_nul, que es dona si no s'ha indicat el codi de contracte del contador
 IF mod_codi_contracte IS NULL THEN
    RAISE exc_codi_contracte_nul;
 END IF;

 -- Excepci exc_contador_no_existeix, que es dona si el contador no existeix
 SELECT COUNT(*) INTO trobats FROM CONTADORS WHERE codi_contracte = mod_codi_contracte;
 IF trobats = 0 THEN
    RAISE exc_contador_no_existeix;
 END IF;

 -- Excepci exc_potencia_nul, que es dona si no s'ha indicat la potncia contractada
 IF mod_potencia IS NULL THEN
    RAISE exc_potencia_nul;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea del contador
 IF mod_adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal del contador
 IF mod_codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci del contador
 IF mod_poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia del contador
 IF mod_provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas del contador
 IF mod_pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;

 -- Excepci exc_any_incorrecte, que es dona si l'any especificat no es troba entre 1850 i 9999.
 IF mod_any_fabricacio < 1850 OR mod_any_fabricacio > 9999
  OR TRUNC(mod_any_fabricacio) <> mod_any_fabricacio THEN
    RAISE exc_any_incorrecte;
 END IF;  

 -- Excepci exc_alta_disp_nul, que es dona si no s'ha indicat si el contador t servei d'alta
 -- disponibilitat
 IF mod_alta_disp IS NULL THEN
    RAISE exc_alta_disp_nul;
 END IF;

 -- Excepci exc_client_nul, que es produeix si no s'ha indicat el NIF del client
 IF mod_client IS NULL THEN
    RAISE exc_client_nul;
 END IF;

 -- Excepci exc_client_no_existeix, que es dona si el NIF indicat no existeix
 SELECT COUNT(*) INTO trobats FROM PERSONES WHERE NIF = mod_client;
 IF trobats = 0 THEN
    RAISE exc_client_no_existeix;
 END IF;

 -- Excepci exc_no_dues_CDs, que es dona si es vol posar a 1 el servei d'alta disponibilitat,
 -- per el contador no est associat a dues o ms centraletes de distribuci
 SELECT COUNT(*) INTO trobats FROM CENTRALETES_DE_CONTADORS WHERE codi_contracte = mod_codi_contracte;
 IF trobats < 2 THEN
    RAISE exc_no_dues_CDs;
 END IF;

 -- Excepci exc_falta_capacitat_CD, que es dona si la centraleta no t prou capacitat lliure,
 -- en el cas que la potncia s'amplii.
 -- La capacitat de la centraleta de distribuci s mensual, i la del contracte en KW. Aix,
 -- per verificar la centraleta t prou capacitat, considerar un mes de 30 dies, s a dir, 720 hores.
 SELECT potencia_contractada INTO pot_vella FROM CONTADORS WHERE codi_contracte = mod_codi_contracte;
 IF mod_potencia > pot_vella THEN
    SELECT codi_central, capacitat_maxima BULK COLLECT INTO taula_temp
     FROM CENTRALS_DE_DISTRIBUCIO
     WHERE codi_central IN (SELECT DISTINCT codi_central FROM CENTRALETES_DE_CONTADORS 
     WHERE codi_contracte = mod_codi_contracte);
    FOR i IN taula_temp.FIRST .. taula_temp.LAST LOOP
       capacitat_ocupada := 720*fn_pot_contr_CD(taula_temp(i).codi_central);
       capacitat_lliure := taula_temp(i).capacitat - capacitat_ocupada;
       IF 720*(mod_potencia-pot_vella) > capacitat_lliure THEN
          RAISE exc_falta_capacitat_CD;
       END IF;
    END LOOP;
 END IF;

 -- Actualitzaci d'un contador en la taula CONTADORS  
 UPDATE CONTADORS SET potencia_contractada = mod_potencia, adrea = mod_adrea,
  codi_postal = mod_codi_postal, poblacio = mod_poblacio, provincia = mod_provincia,
  pais = mod_pais, marca = mod_marca, model_c = mod_model, any_fabricacio = mod_any_fabricacio,
  numero_serie = mod_numero_serie, servei_alta_disponibilitat = mod_alta_disp, client = mod_client
  WHERE codi_contracte = mod_codi_contracte;  
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_codi_contracte_nul THEN 
     resultat := 'ERROR. Cal indicar el codi de contracte del contador.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_contador_no_existeix THEN 
     resultat := 'ERROR. No existeix el contador amb codi de contracte ' || mod_codi_contracte || '.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_potencia_nul THEN 
     resultat := 'ERROR. Cal indicar la potncia contractada pel contador.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_adrea_nul THEN 
     resultat := 'ERROR. Cal indicar l''adrea del contador.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_codi_postal_nul THEN 
     resultat := 'ERROR. Cal indicar el codi postal del contador.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_poblacio_nul THEN 
     resultat := 'ERROR. Cal indicar la poblaci del contador.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_provincia_nul THEN 
     resultat := 'ERROR. Cal indicar la provncia del contador.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_pais_nul THEN 
     resultat := 'ERROR. Cal indicar el pas del contador.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_any_incorrecte THEN 
     resultat := 'ERROR. L''any de fabricaci especificat pel contador s incorrecte.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_alta_disp_nul THEN 
     resultat := 'ERROR. Cal indicar si el contador disposa de Servei d''Alta Disponibilitat.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_no_dues_CDs THEN 
     resultat := 'ERROR. El contador no est associat a almenys dues centraletes de distribuci.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_falta_capacitat_CD THEN 
     resultat := 'ERROR. El contador no est associat a almenys dues centraletes de distribuci.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_client_nul THEN 
     resultat := 'ERROR. Cal indicar la persona titular del contracte.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_client_no_existeix THEN 
     resultat := 'ERROR. La persona amb NIF ' || mod_client || ' no existeix.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una Central de Producci o Distribuci */
 -- Modificaci dels parmetres comuns a qualsevol central, sigui de distribuci o
 -- de producci (del tipus que sigui)
 -- les dades comunes sn l'adrea (adrea, codi postal, poblaci, provncia i pas)
 -- la data de la darrera inspecci tcnica i la capacitat mxima mensual.
CREATE OR REPLACE PROCEDURE pr_mod_dades_comunes_central (mod_codi_central VARCHAR,
 tipus_central VARCHAR,
 mod_adrea VARCHAR, mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR,
 mod_pais VARCHAR, mod_data_inspeccio DATE, mod_capacitat NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
 nom_de_taula VARCHAR(25);
 sql_string VARCHAR (300);
 potencia_necessaria NUMBER;
 capacitat_vella NUMBER;
    
 -- Variables per excepcions 
 exc_codi_central_nul EXCEPTION;
 exc_central_no_existeix EXCEPTION;
 exc_tipus_incorrecte EXCEPTION;
 exc_adrea_nul EXCEPTION;
 exc_codi_postal_nul EXCEPTION;
 exc_poblacio_nul EXCEPTION;
 exc_provincia_nul EXCEPTION;
 exc_pais_nul EXCEPTION;
 exc_capacitat_nul EXCEPTION;
 exc_falta_capacitat_CD EXCEPTION;
 exc_falta_capacitat_CP EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_mod_dades_comunes_central';
 parametres_entrada := 'codi_central = ' || mod_codi_central ||
  '; adrea = ' || mod_adrea || '; codi_postal = ' || mod_codi_postal ||
  '; poblacio = ' || mod_poblacio || '; provincia = ' || mod_provincia ||
  '; pais = ' || mod_pais || '; data_darrera_inspeccio = ' || mod_data_inspeccio ||
  '; capacitat_maxima = ' || TO_CHAR(mod_capacitat) || ';';
  
 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la central
 IF mod_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_no_existeix, que es dona si la central no existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = mod_codi_central;
 IF trobats = 0 THEN
    RAISE exc_central_no_existeix;
 END IF;

 -- Excepci exc_adrea_nul, que es dona si no s'ha indicat l'adrea de la central
 IF mod_adrea IS NULL THEN
    RAISE exc_adrea_nul;
 END IF;

 -- Excepci exc_codi_postal_nul, que es dona si no s'ha indicat el codi postal de la central
 IF mod_codi_postal IS NULL THEN
    RAISE exc_codi_postal_nul;
 END IF;

 -- Excepci exc_poblacio_nul, que es dona si no s'ha indicat la poblaci de la central
 IF mod_poblacio IS NULL THEN
    RAISE exc_poblacio_nul;
 END IF;

 -- Excepci exc_provincia_nul, que es dona si no s'ha indicat la provncia de la central
 IF mod_provincia IS NULL THEN
    RAISE exc_provincia_nul;
 END IF;

 -- Excepci exc_pais_nul, que es dona si no s'ha indicat el pas de la central
 IF mod_pais IS NULL THEN
    RAISE exc_pais_nul;
 END IF;

 -- Excepci exc_capacitat_nul, que es dona si no s'ha indicat la capacitat mensual mxima de la central
 IF mod_capacitat IS NULL THEN
    RAISE exc_capacitat_nul;
 END IF;

 -- Busquem la taula corresponent a la central indicada
 SELECT nom_taula INTO nom_de_taula FROM TAULES_DE_CENTRALS WHERE codi_central = mod_codi_central;

 -- Excepci exc_tipus_incorrecte, que es dona si la central no s del tipus indicat
 IF tipus_central = 'nuclear' THEN
    IF nom_de_taula <> 'CENTRALS_NUCLEARS' THEN
       RAISE exc_tipus_incorrecte;
    END IF;
 END IF;
 IF tipus_central = 'termica' THEN
    IF nom_de_taula <> 'CENTRALS_TERMIQUES' THEN
       RAISE exc_tipus_incorrecte;
    END IF;
 END IF;
 IF tipus_central = 'carbo' THEN
    IF nom_de_taula <> 'CENTRALS_DE_CARBO' THEN
       RAISE exc_tipus_incorrecte;
    END IF;
 END IF;
 IF tipus_central = 'eolica' THEN
    IF nom_de_taula <> 'CENTRALS_EOLIQUES' THEN
       RAISE exc_tipus_incorrecte;
    END IF;
 END IF;
 IF tipus_central = 'solar' THEN
    IF nom_de_taula <> 'CENTRALS_SOLARS' THEN
       RAISE exc_tipus_incorrecte;
    END IF;
 END IF;
 IF tipus_central = 'distribucio' THEN
    IF nom_de_taula <> 'CENTRALS_DE_DISTRIBUCIO' THEN
       RAISE exc_tipus_incorrecte;
    END IF;
 END IF;

 sql_string := 'SELECT capacitat_maxima FROM ' || nom_de_taula || ' WHERE codi_central = :a';
 EXECUTE IMMEDIATE sql_string INTO capacitat_vella USING mod_codi_central;

 -- Si la capacitat es vol disminur, cal comprovar si hi haur prou capacitat per les centraletes
 -- de distribuci, en el cas de les centrals de producci, o pels contadors, en el cas de les
 -- centraletes de distribuci.
 -- Considero un mes de 30 dies, s a dir, 720 hores, per comparar potncia contractada pels contadors
 -- amb capacitat mxima mensual de les centraletes de distribuci.

 IF mod_capacitat < capacitat_vella THEN

    -- Excepci exc_falta_capacitat_CD, que es dona si la capacitat mxima indicada no s suficient per
    -- abastir els contadors ja associats a la centraleta de distribuci.
    IF nom_de_taula = 'CENTRALS_DE_DISTRIBUCIO' THEN
       SELECT SUM(potencia_contractada*fn_frac_pot_contador(codi_contracte))
        INTO potencia_necessaria FROM CONTADORS
        WHERE codi_contracte IN (SELECT DISTINCT codi_contracte FROM CENTRALETES_DE_CONTADORS
        WHERE codi_central = mod_codi_central);
       IF potencia_necessaria > mod_capacitat/720 THEN
          RAISE exc_falta_capacitat_CD;
       END IF;
    -- Excepci exc_falta_capacitat_CP, que es dona si la capacitat mxima indicada no s suficient per
    -- abastir les centraletes de distribuci ja associades a la central de producci.
    ELSE
       SELECT SUM(capacitat_maxima*fn_frac_pot_centraleta(codi_central))
        INTO potencia_necessaria FROM CENTRALS_DE_DISTRIBUCIO -- no s potncia, per aprofito la variable
        WHERE codi_central IN (SELECT DISTINCT codi_central_distribucio FROM LINIES_DE_COMUNICACIO
        WHERE codi_central_produccio = mod_codi_central);
       IF potencia_necessaria > mod_capacitat THEN
          RAISE exc_falta_capacitat_CP;
       END IF;
    END IF;

 END IF;

 -- Actualitzaci de les dades comunes de les centrals en la taula corresponent  
 sql_string := 'UPDATE ' || nom_de_taula || ' SET adrea = :a, codi_postal = :b,poblacio = :c, provincia = :d,
  pais = :e, data_darrera_inspeccio = :f, capacitat_maxima = :g WHERE codi_central = :h';
 EXECUTE IMMEDIATE sql_string USING mod_adrea, mod_codi_postal, mod_poblacio, mod_provincia,
  mod_pais, mod_data_inspeccio, mod_capacitat, mod_codi_central;  
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_codi_central_nul THEN 
     resultat := 'ERROR. Cal indicar el codi de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_central_no_existeix THEN 
     resultat := 'ERROR. No existeix la central amb codi ' || mod_codi_central || '.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_adrea_nul THEN 
     resultat := 'ERROR. Cal indicar l''adrea de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_codi_postal_nul THEN 
     resultat := 'ERROR. Cal indicar el codi postal de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_poblacio_nul THEN 
     resultat := 'ERROR. Cal indicar la poblaci de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_provincia_nul THEN 
     resultat := 'ERROR. Cal indicar la provncia de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_pais_nul THEN 
     resultat := 'ERROR. Cal indicar el pas de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_capacitat_nul THEN 
     resultat := 'ERROR. Cal indicar la capacitat mxima de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_tipus_incorrecte THEN 
     resultat := 'ERROR. La central no s del tipus indicat (' || tipus_central || ').';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_falta_capacitat_CD THEN 
     resultat := 'ERROR. La capacitat mxima indicada no s suficient pels contadors associats.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_falta_capacitat_CP THEN 
     resultat := 'ERROR. La capacitat mxima indicada no s suficient per les centraletes de
      de distribuci associades.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una Central de Producci Nuclear */
 -- Es cridar el procediment pr_mod_dades_comunes_central, pels parmetres de qualsevol central
 -- (inclosa distribuci)
CREATE OR REPLACE PROCEDURE pr_modificacio_central_nuclear (mod_codi_central VARCHAR,
 mod_adrea VARCHAR, mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR,
 mod_pais VARCHAR, mod_data_inspeccio DATE, mod_capacitat_maxima NUMBER,
 mod_energia_minima NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
    
 -- Variables per excepcions
 exc_error_crida_fill EXCEPTION;
 exc_energia_minima_nul EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_modificacio_central_nuclear';
 parametres_entrada := 'codi_central = ' || mod_codi_central ||
  '; adrea = ' || mod_adrea || '; codi_postal = ' || mod_codi_postal ||
  '; poblacio = ' || mod_poblacio || '; provincia = ' || mod_provincia ||
  '; pais = ' || mod_pais || '; data_darrera_inspeccio = ' || mod_data_inspeccio ||
  '; capacitat_maxima = ' || mod_capacitat_maxima ||
  '; energia_minima_funcionament = ' || mod_energia_minima || ';';
  
 -- Excepci exc_energia_minima_nul, que es dona si no s'ha indicat l'energia mnima
 -- per al funcionament de la central nuclear
 IF mod_energia_minima IS NULL THEN
    RAISE exc_energia_minima_nul;
 END IF;

 -- Crida al procediment pr_mod_dades_comunes_central, que ja verifica codi_central, adrea
 -- i capacitat mxima
 pr_mod_dades_comunes_central(mod_codi_central, 'nuclear', mod_adrea, mod_codi_postal,
  mod_poblacio, mod_provincia, mod_pais, mod_data_inspeccio, mod_capacitat_maxima, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Actualitzaci de les dades prpies d'una central nuclear  
 UPDATE CENTRALS_NUCLEARS SET energia_minima_de_funcionament = mod_energia_minima
  WHERE codi_central = mod_codi_central;
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_energia_minima_nul THEN 
     resultat := 'ERROR. Cal indicar l''energia mnima per al funcionament de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_error_crida_fill THEN 
     resultat := 'ERROR. Error en la crida a un procediment niuat.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una Central de Producci Trmica */
 -- Es cridar el procediment pr_mod_dades_comunes_central, pels parmetres de qualsevol central
 -- (inclosa distribuci)
CREATE OR REPLACE PROCEDURE pr_modificacio_central_termica (mod_codi_central VARCHAR,
 mod_adrea VARCHAR, mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR,
 mod_pais VARCHAR, mod_data_inspeccio DATE, mod_capacitat_maxima NUMBER,
 mod_emissions_CO2 NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables per excepcions
 exc_error_crida_fill EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_modificacio_central_termica';
 parametres_entrada := 'codi_central = ' || mod_codi_central ||
  '; adrea = ' || mod_adrea || '; codi_postal = ' || mod_codi_postal ||
  '; poblacio = ' || mod_poblacio || '; provincia = ' || mod_provincia ||
  '; pais = ' || mod_pais || '; data_darrera_inspeccio = ' || mod_data_inspeccio ||
  '; capacitat_maxima = ' || mod_capacitat_maxima ||
  '; emissions_CO2 = ' || mod_emissions_CO2 || ';';

 -- Crida al procediment pr_mod_dades_comunes_central, que ja verifica codi_central, adrea
 -- i capacitat mxima
 pr_mod_dades_comunes_central(mod_codi_central, 'termica', mod_adrea, mod_codi_postal,
  mod_poblacio, mod_provincia, mod_pais, mod_data_inspeccio, mod_capacitat_maxima, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Actualitzaci de les dades prpies d'una central nuclear  
 UPDATE CENTRALS_TERMIQUES SET emissions_CO2 = mod_emissions_CO2
  WHERE codi_central = mod_codi_central;
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_error_crida_fill THEN 
     resultat := 'ERROR. Error en la crida a un procediment niuat.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una Central de Producci de Carb */
 -- Es cridar el procediment pr_mod_dades_comunes_central, pels parmetres de qualsevol central
 -- (inclosa distribuci)
CREATE OR REPLACE PROCEDURE pr_modificacio_central_carbo (mod_codi_central VARCHAR,
 mod_adrea VARCHAR, mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR,
 mod_pais VARCHAR, mod_data_inspeccio DATE, mod_capacitat_maxima NUMBER,
 mod_emissions_CO2 NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables per excepcions
 exc_error_crida_fill EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_modificacio_central_carbo';
 parametres_entrada := 'codi_central = ' || mod_codi_central ||
  '; adrea = ' || mod_adrea || '; codi_postal = ' || mod_codi_postal ||
  '; poblacio = ' || mod_poblacio || '; provincia = ' || mod_provincia ||
  '; pais = ' || mod_pais || '; data_darrera_inspeccio = ' || mod_data_inspeccio ||
  '; capacitat_maxima = ' || mod_capacitat_maxima ||
  '; emissions_CO2 = ' || mod_emissions_CO2 || ';';

 -- Crida al procediment pr_mod_dades_comunes_central, que ja verifica codi_central i adrea
 pr_mod_dades_comunes_central(mod_codi_central, 'carbo', mod_adrea, mod_codi_postal,
  mod_poblacio, mod_provincia, mod_pais, mod_data_inspeccio, mod_capacitat_maxima, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Actualitzaci de les dades prpies d'una central nuclear  
 UPDATE CENTRALS_DE_CARBO SET emissions_CO2 = mod_emissions_CO2
  WHERE codi_central = mod_codi_central;
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_error_crida_fill THEN 
     resultat := 'ERROR. Error en la crida a un procediment niuat.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una Central de Producci Elica */
 -- Es cridar el procediment pr_mod_dades_comunes_central, pels parmetres de qualsevol central
 -- (inclosa distribuci)
CREATE OR REPLACE PROCEDURE pr_modificacio_central_eolica (mod_codi_central VARCHAR,
 mod_adrea VARCHAR, mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR,
 mod_pais VARCHAR, mod_data_inspeccio DATE, mod_capacitat_maxima NUMBER,
 mod_numero_molins INTEGER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables per excepcions
 exc_error_crida_fill EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_modificacio_central_eolica';
 parametres_entrada := 'codi_central = ' || mod_codi_central ||
  '; adrea = ' || mod_adrea || '; codi_postal = ' || mod_codi_postal ||
  '; poblacio = ' || mod_poblacio || '; provincia = ' || mod_provincia ||
  '; pais = ' || mod_pais || '; data_darrera_inspeccio = ' || mod_data_inspeccio ||
  '; capacitat_maxima = ' || mod_capacitat_maxima ||
  '; numero_molins = ' || mod_numero_molins || ';';

 -- Crida al procediment pr_mod_dades_comunes_central, que ja verifica codi_central i adrea
 pr_mod_dades_comunes_central(mod_codi_central, 'eolica', mod_adrea, mod_codi_postal,
  mod_poblacio, mod_provincia, mod_pais, mod_data_inspeccio, mod_capacitat_maxima, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Actualitzaci de les dades prpies d'una central nuclear  
 UPDATE CENTRALS_EOLIQUES SET numero_de_molins = mod_numero_molins
  WHERE codi_central = mod_codi_central;
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_error_crida_fill THEN 
     resultat := 'ERROR. Error en la crida a un procediment niuat.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una Central de Producci Solar */
 -- Es cridar el procediment pr_mod_dades_comunes_central, pels parmetres de qualsevol central
 -- (inclosa distribuci)
CREATE OR REPLACE PROCEDURE pr_modificacio_central_solar (mod_codi_central VARCHAR,
 mod_adrea VARCHAR, mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR,
 mod_pais VARCHAR, mod_data_inspeccio DATE, mod_capacitat_maxima NUMBER,
 mod_minim_panells INTEGER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
    
 -- Variables per excepcions
 exc_error_crida_fill EXCEPTION;
 exc_minim_panells_nul EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_modificacio_central_solar';
 parametres_entrada := 'codi_central = ' || mod_codi_central ||
  '; adrea = ' || mod_adrea || '; codi_postal = ' || mod_codi_postal ||
  '; poblacio = ' || mod_poblacio || '; provincia = ' || mod_provincia ||
  '; pais = ' || mod_pais || '; data_darrera_inspeccio = ' || mod_data_inspeccio ||
  '; capacitat_maxima = ' || mod_capacitat_maxima ||
  '; nombre_minim_panells_funcionament = ' || mod_minim_panells || ';';
  
 -- Excepci exc_minim_panells_nul, que es dona si no s'ha indicat el nombre mnim
 -- de panells per al funcionament de la central solar
 IF mod_minim_panells IS NULL THEN
    RAISE exc_minim_panells_nul;
 END IF;

 -- Crida al procediment pr_mod_dades_comunes_central, que ja verifica codi_central i adrea
 pr_mod_dades_comunes_central(mod_codi_central, 'solar', mod_adrea, mod_codi_postal,
  mod_poblacio, mod_provincia, mod_pais, mod_data_inspeccio, mod_capacitat_maxima, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Actualitzaci de les dades prpies d'una central nuclear  
 UPDATE CENTRALS_SOLARS SET numero_panells_funcionament = mod_minim_panells
  WHERE codi_central = mod_codi_central;
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_minim_panells_nul THEN 
     resultat := 'ERROR. Cal indicar el nombre mnim de panells per al funcionament de la central.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_error_crida_fill THEN 
     resultat := 'ERROR. Error en la crida a un procediment niuat.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una Central de Distribuci */
 -- Es cridar el procediment pr_mod_dades_comunes_central, pels parmetres de
 -- qualsevol central (inclosa distribuci)
CREATE OR REPLACE PROCEDURE pr_mod_central_distribucio (mod_codi_central VARCHAR,
 mod_adrea VARCHAR, mod_codi_postal NUMBER, mod_poblacio VARCHAR, mod_provincia VARCHAR,
 mod_pais VARCHAR, mod_data_inspeccio DATE, mod_capacitat_maxima NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
    
 -- Variables per excepcions
 exc_error_crida_fill EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_mod_central_distribucio';
 parametres_entrada := 'codi_central = ' || mod_codi_central ||
  '; adrea = ' || mod_adrea || '; codi_postal = ' || mod_codi_postal ||
  '; poblacio = ' || mod_poblacio || '; provincia = ' || mod_provincia ||
  '; pais = ' || mod_pais || '; data_darrera_inspeccio = ' || mod_data_inspeccio ||
  '; capacitat_maxima = ' || mod_capacitat_maxima || ';';

 -- Crida al procediment pr_mod_dades_comunes_central, que ja verifica codi_central i adrea
 pr_mod_dades_comunes_central(mod_codi_central, 'distribucio', mod_adrea, mod_codi_postal,
  mod_poblacio, mod_provincia, mod_pais, mod_data_inspeccio, mod_capacitat_maxima, resultat);
 IF resultat <> 'OK' THEN
    RAISE exc_error_crida_fill;
 END IF;

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_error_crida_fill THEN 
     resultat := 'ERROR. Error en la crida a un procediment niuat.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una Lnia de Comunicaci */
 -- Noms es permetr modificar la capacitat mxima
CREATE OR REPLACE PROCEDURE pr_mod_linia_comunicacio (mod_codi_linia VARCHAR,
 mod_capacitat NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
    
 -- Variables de consulta
 trobats INTEGER;
 capacitat_Central NUMBER;

 -- Variables per excepcions
 exc_codi_linia_nul EXCEPTION;
 exc_linia_no_existeix EXCEPTION;
 exc_capacitat_nul EXCEPTION;
 exc_falta_capacitat_linia EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_mod_linia_comunicacio';
 parametres_entrada := 'codi_linia = ' || mod_codi_linia ||
  '; capacitat_maxima = ' || mod_capacitat || ';';
  
 -- Excepci exc_codi_linia_nul, que es dona si no s'ha indicat el codi de la lnia
 IF mod_codi_linia IS NULL THEN
    RAISE exc_codi_linia_nul;
 END IF;

 -- Excepci exc_linia_no_existeix, que es dona si la lnia de comunicaci no existeix
 SELECT COUNT(*) INTO trobats FROM LINIES_DE_COMUNICACIO WHERE codi_linia = mod_codi_linia;
 IF trobats = 0 THEN
    RAISE exc_linia_no_existeix;
 END IF;

 -- Excepci exc_capacitat_nul, que es dona si no s'ha indicat la capacitat mxima
 -- de la central de distribuci
 IF mod_capacitat IS NULL THEN
    RAISE exc_capacitat_nul;
 END IF;

 -- Excepci exc_falta_capacitat_linia, que es dona si la capacitat indicada per la lnia de comunicaci
 -- no es suficient per abastir la centraleta de distribuci.
 -- La capacitat de la centraleta de distribuci s mensual, i la de la lnia en KW. Aix, per verificar
 -- si la lnia t prou capacitat, considerar un mes de 30 dies, s a dir, 720 hores.
 SELECT capacitat_maxima INTO capacitat_Central FROM CENTRALS_DE_DISTRIBUCIO
  WHERE codi_central = (SELECT codi_central_distribucio FROM LINIES_DE_COMUNICACIO
   WHERE codi_linia = mod_codi_linia);
 IF mod_capacitat < capacitat_Central/720 THEN
    RAISE exc_falta_capacitat_linia;
 END IF;

 -- Actualitzaci de les dades d'una lnia de comunicaci  
 UPDATE LINIES_DE_COMUNICACIO SET capacitat_maxima = mod_capacitat
  WHERE codi_linia = mod_codi_linia;
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_codi_linia_nul THEN 
     resultat := 'ERROR. Cal indicar el codi de la lnia de comunicaci.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_linia_no_existeix THEN 
     resultat := 'ERROR. No existeix la lnia de comunicaci amb codi ' || mod_codi_linia || '.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_capacitat_nul THEN 
     resultat := 'ERROR. Cal indicar la capacitat mxima de la lnia de comunicaci.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_falta_capacitat_linia THEN 
     resultat := 'ERROR. La capacitat de la lnia no s suficient per abastir la centraleta de distribuci.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
END;
/



/* Modificaci d'una relaci Contador-Centraleta de Distribuci */
/* Abans de la modificaci es comprobar la capacitat "lliure" de la centraleta nova, i
   si no s suficient, es produr una excepci */
 -- Es permetr modificar la centraleta de distribuci
CREATE OR REPLACE PROCEDURE pr_mod_centraleta_contracte (mod_codi_contracte VARCHAR,
 mod_codi_central_vella NUMBER, mod_codi_central_nova NUMBER, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
    
 -- Variables de consulta
 trobats INTEGER;
 capacitat_total NUMBER;
 capacitat_ocupada NUMBER;
 capacitat_lliure NUMBER;
 potencia_contador NUMBER;
 estat_baixa NUMBER;

 -- Variables per excepcions
 exc_codi_contracte_nul EXCEPTION;
 exc_contracte_baixa EXCEPTION;
 exc_codi_central_vella_nul EXCEPTION;
 exc_central_cont_no_existeix EXCEPTION;
 exc_codi_central_nova_nul EXCEPTION;
 exc_central_nova_no_existeix EXCEPTION;
 exc_falta_capacitat EXCEPTION;
  
BEGIN

 -- Inicialitza variables de la taula LOGS
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_mod_centraleta_contracte';
 parametres_entrada := 'codi_contracte = ' || mod_codi_contracte ||
  '; codi_centraleta_vella = ' || mod_codi_central_vella ||
  '; codi_centraleta_nova = ' || mod_codi_central_nova || ';';
  
 -- Excepci exc_codi_contracte_nul, que es dona si no s'ha indicat el codi de contracte
 IF mod_codi_contracte IS NULL THEN
    RAISE exc_codi_contracte_nul;
 END IF;

 -- Excepci exc_contracte_baixa, que es dona si el contador est donat de baixa
 SELECT baixa INTO estat_baixa FROM CONTADORS WHERE codi_contracte = mod_codi_contracte;
 IF estat_baixa = 1 THEN
    RAISE exc_contracte_baixa;
 END IF;

 -- Excepci exc_codi_central_vella_nul, que es dona si no s'ha indicat el codi de la
 -- centraleta de distribuci vella
 IF mod_codi_central_vella IS NULL THEN
    RAISE exc_codi_central_vella_nul;
 END IF;

 -- Excepci exc_central_cont_no_existeix, que es dona si no existeix
 -- la relaci entre el contador i la centraleta de distribuci
 SELECT COUNT(*) INTO trobats FROM CENTRALETES_DE_CONTADORS WHERE
  (codi_contracte = mod_codi_contracte AND codi_central = mod_codi_central_vella);
 IF trobats = 0 THEN
    RAISE exc_central_cont_no_existeix;
 END IF;

 -- Excepci exc_codi_central_nova_nul, que es dona si no s'ha indicat el codi de la
 -- centraleta de distribuci nova
 IF mod_codi_central_nova IS NULL THEN
    RAISE exc_codi_central_nova_nul;
 END IF;

 -- Excepci exc_central_nova_no_existeix, que es dona si no existeix
 -- la centraleta de distribuci nova
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = mod_codi_central_nova;
 IF trobats = 0 THEN
    RAISE exc_central_nova_no_existeix;
 END IF;

 -- Excepci exc_falta_capacitat, que es dona si la centraleta no t prou capacitat lliure
 SELECT capacitat_maxima INTO capacitat_total FROM CENTRALS_DE_DISTRIBUCIO
  WHERE codi_central = mod_codi_central_nova;
 SELECT SUM(potencia_contractada) INTO capacitat_ocupada FROM CONTADORS c WHERE EXISTS
  (SELECT * FROM CENTRALETES_DE_CONTADORS r WHERE r.codi_central = mod_codi_central_nova AND
  r.codi_contracte = c.codi_contracte);
 capacitat_lliure := capacitat_total - capacitat_ocupada;
 SELECT potencia_contractada INTO potencia_contador FROM CONTADORS
  WHERE codi_contracte = mod_codi_contracte;
 IF potencia_contador > capacitat_lliure THEN
    RAISE exc_falta_capacitat;
 END IF;

 -- Actualitzaci de les dades d'una lnia de comunicaci  
 UPDATE CENTRALETES_DE_CONTADORS SET codi_central = mod_codi_central_nova
  WHERE codi_central = mod_codi_central_vella;
 resultat := 'OK';

 -- Inserci de la informaci de la crida a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

EXCEPTION
  WHEN exc_codi_contracte_nul THEN 
     resultat := 'ERROR. Cal indicar el codi de contracte del contador.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_contracte_baixa THEN 
     resultat := 'ERROR. El contracte ' || mod_codi_contracte || ' est donat de baixa.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_codi_central_vella_nul THEN 
     resultat := 'ERROR. Cal indicar el codi de la centraleta de distribuci antiga.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_central_cont_no_existeix THEN 
     resultat := 'ERROR. No existeix la relaci entre el contracte ' || mod_codi_contracte ||
      ' i la centraleta de distribuci amb codi ' || mod_codi_central_vella || '.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_codi_central_nova_nul THEN 
     resultat := 'ERROR. Cal indicar el codi de la nova centraleta de distribuci.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
  WHEN exc_central_nova_no_existeix THEN 
     resultat := 'ERROR. No existeix la centraleta de distribuci amb codi '
      || mod_codi_central_nova || '.';    
     INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
     COMMIT;
 WHEN exc_falta_capacitat THEN 
    resultat := 'ERROR. La centraleta de distribuci no t prou capacitat lliure.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/




/* Procediments de Baixa */

/* Baixa d'una Persona */
CREATE OR REPLACE PROCEDURE pr_baixa_persona (baixa_NIF VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_NIF_nul EXCEPTION;
 exc_NIF_no_existeix EXCEPTION;
 exc_contractes_NIF EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_baixa_persona';
 parametres_entrada := 'NIF = ' || baixa_NIF || ';';

 -- Excepci exc_NIF_nul, que es dona si no s'ha indicat el valor del NIF
 IF baixa_NIF IS NULL THEN
    RAISE exc_NIF_nul;
 END IF;

 -- Excepci exc_NIF_no_existeix, que es dona si la persona (NIF) no existeix
 SELECT COUNT(*) INTO trobats FROM PERSONES WHERE NIF = baixa_NIF;
 IF trobats = 0 THEN
    RAISE exc_NIF_no_existeix;
 END IF;
  
 -- Excepci exc_contractes_NIF, que es dona si la persona (NIF) t contractes actius,
 -- per la qual cosa no es pot esborrar.
 SELECT COUNT(*) INTO trobats FROM CONTADORS WHERE client = baixa_NIF AND baixa = 0;
 IF trobats > 0 THEN
    RAISE exc_contractes_NIF;
 END IF;

 -- Supressi de la persona en la taula PERSONES
 DELETE FROM PERSONES WHERE NIF = baixa_NIF;
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_NIF_nul THEN 
    resultat := 'ERROR. Cal introduir el NIF de la persona.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_NIF_no_existeix THEN 
    resultat := 'ERROR. No existeix una persona amb el NIF ' || baixa_NIF || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contractes_NIF THEN 
    resultat := 'ERROR. La persona t contractes associats i no es pot esborrar.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Baixa d'un Contador */
/* 
   Implicar la supressi de les relacions del contador en la taula CENTRALETES_DE_CONTADORS,
   El camp baixa del contador passar a 1, per no s'esborraran les lectures de contador
   en la taula LECTURES
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
CREATE OR REPLACE PROCEDURE pr_baixa_contador (baixa_codi_contracte VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
 estat_baixa NUMBER;
    
 -- Variables per excepcions 
 exc_codi_contracte_nul EXCEPTION;
 exc_contracte_no_existeix EXCEPTION;
 exc_contracte_baixa EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_baixa_contador';
 parametres_entrada := 'codi_contracte = ' || baixa_codi_contracte || ';';

 -- Excepci exc_codi_contracte_nul, que es dona si no s'ha indicat el codi de contracte
 IF baixa_codi_contracte IS NULL THEN
    RAISE exc_codi_contracte_nul;
 END IF;

 -- Excepci exc_contracte_no_existeix, que es dona si el contador no existeix
 SELECT COUNT(*) INTO trobats FROM CONTADORS WHERE codi_contracte = baixa_codi_contracte;
 IF trobats = 0 THEN
    RAISE exc_contracte_no_existeix;
 END IF;

 -- Excepci exc_contracte_baixa, que es dona si el contador ja est donat de baixa
 SELECT baixa INTO estat_baixa FROM CONTADORS WHERE codi_contracte = baixa_codi_contracte;
 IF estat_baixa = 1 THEN
    RAISE exc_contracte_baixa;
 END IF;

 -- Supressi de les relacions del contador amb les centraletes de distribuci
 DELETE FROM CENTRALETES_DE_CONTADORS WHERE codi_contracte = baixa_codi_contracte;

 -- En la taula CONTADORS posem el camp baixa del contador a 1
 UPDATE CONTADORS SET baixa = 1 WHERE codi_contracte = baixa_codi_contracte;
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_contracte_nul THEN 
    resultat := 'ERROR. Cal introduir el codi de contracte del contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contracte_no_existeix THEN 
    resultat := 'ERROR. No existeix un contador amb el codi de contracte '
     || baixa_codi_contracte || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_contracte_baixa THEN 
    resultat := 'ERROR. El contracte ' || baixa_codi_contracte || ' ja est donat de baixa.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Baixa d'una Central (de distribuci o de producci, del tipus que sigui) */
/* 
   Implicar la supressi de les relacions de la central en la taula CENTRALETES_DE_CONTADORS,
   aix com la supressi de les seva fila en la taula TAULES_DE_CENTRALS
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
CREATE OR REPLACE PROCEDURE pr_baixa_central (baixa_codi_central VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
 nom_de_taula VARCHAR(25);
 sql_string VARCHAR (300);
    
 -- Variables per excepcions 
 exc_codi_central_nul EXCEPTION;
 exc_central_no_existeix EXCEPTION;
 exc_te_contadors EXCEPTION;
 exc_te_centraletes EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_baixa_central';
 parametres_entrada := 'codi_central = ' || baixa_codi_central || ';';

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la central
 IF baixa_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_central_no_existeix, que es dona si la central no existeix
 SELECT COUNT(*) INTO trobats FROM TAULES_DE_CENTRALS WHERE codi_central = baixa_codi_central;
 IF trobats = 0 THEN
    RAISE exc_central_no_existeix;
 END IF;
  
 -- Busquem la taula corresponent a la central indicada
 SELECT nom_taula INTO nom_de_taula FROM TAULES_DE_CENTRALS WHERE codi_central = baixa_codi_central;

 -- Excepci exc_te_contadors, que es dona en cas que sigui una centraleta de distribuci,
 -- si t contadors que en depenen.
 IF nom_de_taula = 'CENTRALS_DE_DISTRIBUCIO' THEN
    SELECT COUNT(*) INTO trobats FROM CENTRALETES_DE_CONTADORS
     WHERE codi_central = baixa_codi_central;
    IF trobats > 0 THEN
       RAISE exc_te_contadors;
    END IF;
 -- Excepci exc_te_centraletes, que es dona en cas que sigui una central de producci,
 -- si t centraletes de distribuci que en depenen.
 ELSE
    SELECT COUNT(*) INTO trobats FROM LINIES_DE_COMUNICACIO
     WHERE codi_central_produccio = baixa_codi_central;
    IF trobats > 0 THEN
       RAISE exc_te_centraletes;
    END IF;
 END IF;

 -- Supressi de la central en la taula corresponent  
 sql_string := 'DELETE FROM ' || nom_de_taula || ' WHERE codi_central = :a';
 EXECUTE IMMEDIATE sql_string USING baixa_codi_central;

 -- Supressi de la central en la taula TAULES_DE_CENTRALS
 DELETE FROM TAULES_DE_CENTRALS WHERE codi_central = baixa_codi_central;
 
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal introduir el codi de la central.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_central_no_existeix THEN 
    resultat := 'ERROR. No existeix una central amb el codi ' || baixa_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_te_contadors THEN 
    resultat := 'ERROR. La centraleta de distribuci t contadors associats.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_te_centraletes THEN 
    resultat := 'ERROR. La central de producci t centraletes de distribuci associades.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Baixa d'una Lnia de Comunicaci */
CREATE OR REPLACE PROCEDURE pr_baixa_linia (baixa_codi_linia VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_linia_nul EXCEPTION;
 exc_linia_no_existeix EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_baixa_linia';
 parametres_entrada := 'codi_linia = ' || baixa_codi_linia || ';';

 -- Excepci exc_codi_linia_nul, que es dona si no s'ha indicat el codi de la lnia de comunicaci
 IF baixa_codi_linia IS NULL THEN
    RAISE exc_codi_linia_nul;
 END IF;

 -- Excepci exc_linia_no_existeix, que es dona si la lnia de comunicaci no existeix
 SELECT COUNT(*) INTO trobats FROM LINIES_DE_COMUNICACIO WHERE codi_linia = baixa_codi_linia;
 IF trobats = 0 THEN
    RAISE exc_linia_no_existeix;
 END IF;
  
 -- Supressi de la linia en la taula LINIES_DE_COMUNICACIO
 DELETE FROM LINIES_DE_COMUNICACIO WHERE codi_linia = baixa_codi_linia;
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_linia_nul THEN 
    resultat := 'ERROR. Cal introduir el codi de la lnia de comunicaci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_linia_no_existeix THEN 
    resultat := 'ERROR. No existeix una lnia de comunicaci amb el codi '
     || baixa_codi_linia || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/



/* Baixa d'una relaci Centraleta de Distribuci - Contador */
CREATE OR REPLACE PROCEDURE pr_baixa_centraleta_contador (baixa_codi_contracte VARCHAR,
 baixa_codi_central VARCHAR, resultat OUT CLOB) IS
  
 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;
  
 -- Variables de consulta
 trobats INTEGER;
    
 -- Variables per excepcions 
 exc_codi_contracte_nul EXCEPTION;
 exc_codi_central_nul EXCEPTION;
 exc_relacio_no_existeix EXCEPTION;
  
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_baixa_centraleta_contador';
 parametres_entrada := 'codi_contracte = ' || baixa_codi_contracte ||
  'codi_central = ' || baixa_codi_central ||';';

 -- Excepci exc_codi_contracte_nul, que es dona si no s'ha indicat el codi de contracte
 IF baixa_codi_contracte IS NULL THEN
    RAISE exc_codi_contracte_nul;
 END IF;

 -- Excepci exc_codi_central_nul, que es dona si no s'ha indicat el codi de la centraleta
 IF baixa_codi_central IS NULL THEN
    RAISE exc_codi_central_nul;
 END IF;

 -- Excepci exc_relacio_no_existeix, que es dona si la lnia de comunicaci no existeix
 SELECT COUNT(*) INTO trobats FROM CENTRALETES_DE_CONTADORS WHERE codi_central = baixa_codi_central
  AND codi_contracte = baixa_codi_contracte;
 IF trobats = 0 THEN
    RAISE exc_relacio_no_existeix;
 END IF;
  
 -- Supressi de la persona en la taula PERSONES
 DELETE FROM CENTRALETES_DE_CONTADORS WHERE codi_central = baixa_codi_central AND
  codi_contracte = baixa_codi_contracte;
 resultat := 'OK';

 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;
  
EXCEPTION
 WHEN exc_codi_contracte_nul THEN 
    resultat := 'ERROR. Cal introduir el codi de contracte del contador.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_codi_central_nul THEN 
    resultat := 'ERROR. Cal introduir el codi de la centraleta de distribuci.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
 WHEN exc_relacio_no_existeix THEN 
    resultat := 'ERROR. No existeix una associaci entre el contador amb codi de contracte '
     || baixa_codi_contracte || ' i la centraleta de distribuci amb codi '
     || baixa_codi_central || '.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);    
    COMMIT;
END;
/