/*	Autor: David Malet
	Data: 29-04-2012
	Projecte: Sistema de Control Energtic
	Revisi: 1
	Base de Dades: Oracle 10g
*/


/* Procediments de Consulta

/* Procediment que, donada una Ciutat i una Data, retorna un llistat amb els contadors
   que han tingut un consum superior al 80% de la mitja (en la ciutat i en la data) */
create or replace PROCEDURE pr_contadors_mes_80pcnt_mitja (ciutat VARCHAR, data_consulta DATE,
 refcursor OUT SYS_REFCURSOR, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 mes_lectura NUMBER;
 any_lectura NUMBER(4);
 trobats INTEGER;
 consum_total NUMBER;
 consum_mig NUMBER;
 percentatge NUMBER;

 -- Variables per excepcions 
 exc_no_contadors EXCEPTION;

 -- resultats
 linia sortida80pcnt%rowtype;
    
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_contadors_mes_80pcnt_mitja';
 parametres_entrada := 'Ciutat = ' || ciutat || '; Data = ' || TO_CHAR(data_consulta) || ';';

 mes_lectura := EXTRACT(MONTH FROM data_consulta);
 any_lectura := EXTRACT(YEAR FROM data_consulta);

 -- Excepci exc_no_contadors, que es dona si no hi ha lectures de cap contador
 -- per la ciutat i la data donats
 SELECT COUNT(*) INTO trobats FROM LECTURES
  WHERE EXTRACT(MONTH FROM data_lectura) = mes_lectura
  AND EXTRACT(YEAR FROM data_lectura) = any_lectura
  AND fn_poblacio_contador(codi_contracte) = ciutat;
 IF trobats = 0 THEN
    RAISE exc_no_contadors;
 END IF;

 -- Obtenci del consum mig
  SELECT SUM(fn_consum_mes(codi_contracte, data_consulta)) INTO consum_total FROM LECTURES
   WHERE EXTRACT(MONTH FROM data_lectura) = mes_lectura
   AND EXTRACT(YEAR FROM data_lectura) = any_lectura
   AND fn_poblacio_contador(codi_contracte) = ciutat;
 consum_mig := consum_total/trobats;

 OPEN refcursor FOR
  SELECT c.codi_contracte, c.potencia_contractada,
   fn_consum_mes(l.codi_contracte,data_consulta)*100/consum_mig AS percentatge 
   FROM CONTADORS c, LECTURES l
   WHERE EXTRACT(MONTH FROM l.data_lectura) = mes_lectura
   AND EXTRACT(YEAR FROM l.data_lectura) = any_lectura
   AND fn_poblacio_contador(l.codi_contracte) = ciutat
   AND c.codi_contracte = l.codi_contracte
   AND fn_consum_mes(l.codi_contracte,data_consulta) > 0.8*consum_mig
   ORDER BY percentatge;
  
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

 -- mostra els resultats
 LOOP
    FETCH refcursor INTO linia;
    EXIT WHEN refcursor%NOTFOUND;
    DBMS_OUTPUT.PUT_LINE(linia.contador || ' - ' || TO_CHAR(linia.potencia)
     || ' - ' || TO_CHAR(linia.percentatge));
 END LOOP;
 CLOSE refcursor;
  
EXCEPTION
 WHEN exc_no_contadors THEN 
    resultat := 'ERROR. No hi ha lectures per aquesta ciutat i aquesta data.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
END;
/




/* Procediment que proporciona un llistat amb les 10 centraletes de distribuci
   que emeten ms energia anualment */
create or replace PROCEDURE pr_10_CDs_mes_energia (refcursor OUT SYS_REFCURSOR,
 resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;

 -- Variables per excepcions 
 exc_no_centraletes EXCEPTION;

 -- resultats
 linia sortida10centraletes%rowtype;
    
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_10_CDs_mes_energia';
 parametres_entrada := '';

 -- Excepci exc_no_centraletes, que es dona si no hi ha cap centraleta de distribuci
 SELECT COUNT(*) INTO trobats FROM CENTRALS_DE_DISTRIBUCIO;
 IF trobats = 0 THEN
    RAISE exc_no_centraletes;
 END IF;

 OPEN refcursor FOR
  SELECT * FROM (
   SELECT DISTINCT c.codi_central, c.adrea, c.codi_postal, c.poblacio, c.provincia, c.pais,
   12*AVG(h.energia_absoluta) OVER (PARTITION BY c.codi_central) AS energia, c.capacitat_maxima
   FROM CENTRALS_DE_DISTRIBUCIO c, CD_HISTORIC_EMISSIO h
   WHERE c.codi_central = h.codi_central
   ORDER BY energia DESC)
  WHERE ROWNUM < 11;
  
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

 -- mostra els resultats
 LOOP
    FETCH refcursor INTO linia;
    EXIT WHEN refcursor%NOTFOUND;
    DBMS_OUTPUT.PUT_LINE(linia.codi || ' - ' || linia.adrea || ' - ' || TO_CHAR(linia.codi_postal)
     || ' - ' || linia.poblacio || ' - ' || linia.provincia
     || ' - ' || linia.pais || ' - ' || TO_CHAR(linia.energia_emesa)
     || ' - ' || TO_CHAR(linia.capacitat_maxima));
 END LOOP;
 CLOSE refcursor;
  
EXCEPTION
 WHEN exc_no_centraletes THEN 
    resultat := 'ERROR. No hi ha centraletes de distribuci en la base de dades.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
END;
/





/* Procediment que proporciona un llistat amb les 10 lnies de comunicaci
   ms carregades, en relaci a la seva capacitat mxima.
   Es podria calcular en funci dels consums dels contadors associats o b
   en funci de les potncies contractades d'aquests contadors.
   Com que volem saber quan es podria ampliar, implementarem la segona opci. */
create or replace PROCEDURE pr_10_linies_mes_carregades (refcursor OUT SYS_REFCURSOR,
 resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;

 -- Variables per excepcions 
 exc_no_linies EXCEPTION;

 -- resultats
 linia sortida10linies%rowtype;
    
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_10_linies_mes_carregades';
 parametres_entrada := '';

 -- Excepci exc_no_centraletes, que es dona si no hi ha cap centraleta de distribuci
 SELECT COUNT(*) INTO trobats FROM LINIES_DE_COMUNICACIO;
 IF trobats = 0 THEN
    RAISE exc_no_linies;
 END IF;

 OPEN refcursor FOR
  SELECT * FROM (
   SELECT DISTINCT codi_linia, fn_pot_contr_CD(codi_central_distribucio)/capacitat_maxima AS carrega,
   capacitat_maxima-fn_pot_contr_CD(codi_central_distribucio)
   FROM LINIES_DE_COMUNICACIO ORDER BY carrega DESC)
  WHERE ROWNUM < 11;
  
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

 -- mostra els resultats
 LOOP
    FETCH refcursor INTO linia;
    EXIT WHEN refcursor%NOTFOUND;
    DBMS_OUTPUT.PUT_LINE(linia.codi || ' - ' || TO_CHAR(linia.carrega)
     || ' - ' || TO_CHAR(linia.ampliacio));
 END LOOP;
 CLOSE refcursor;
  
EXCEPTION
 WHEN exc_no_linies THEN 
    resultat := 'ERROR. No hi ha lnies de comunicaci en la base de dades.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
END;
/




/* Procediment que proporciona un llistat amb els clients que tenen contractat el
   Servei d'Alta Disponibilitat. */
create or replace PROCEDURE pr_clients_alta_disp (refcursor OUT SYS_REFCURSOR,
 resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;

 -- Variables per excepcions 
 exc_no_clients EXCEPTION;

 -- resultats
 linia sortidaAltaDisp%rowtype;
    
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_clients_alta_disp';
 parametres_entrada := '';

 -- Excepci exc_no_clients, que es dona si no hi ha cap client amb Servei d'Alta Disponibilitat
 SELECT COUNT(*) INTO trobats FROM CONTADORS WHERE servei_alta_disponibilitat = 1;
 IF trobats = 0 THEN
    RAISE exc_no_clients;
 END IF;

 OPEN refcursor FOR
  SELECT client, codi_contracte, model_c FROM CONTADORS WHERE servei_alta_disponibilitat = 1
   AND baixa = 0;
  
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

 -- mostra els resultats
 LOOP
    FETCH refcursor INTO linia;
    EXIT WHEN refcursor%NOTFOUND;
    DBMS_OUTPUT.PUT_LINE(linia.nif || ' - ' || linia.codi || ' - ' || linia.model_c);
 END LOOP;
 CLOSE refcursor;
  
EXCEPTION
 WHEN exc_no_clients THEN 
    resultat := 'ERROR. No hi ha cap client amb el Servei d''Alta Disponibilitat contractat.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
END;
/




/* Procediment que, donades una Central de Producci, una data d'inici i una data de fi,
   retorna un llistat amb els consums dels contadors que depenen de la central i
   l'energia total produda per la central. */
create or replace PROCEDURE pr_consum_CP (central VARCHAR, data_inici DATE, data_fi DATE,
 refcursor OUT SYS_REFCURSOR, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;

 -- Variables per excepcions 
 exc_no_contadors EXCEPTION;
 exc_dates_malament EXCEPTION;

 -- resultats
 linia sortidaConsumCP%rowtype;
    
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_consum_CP';
 parametres_entrada := 'Central de Produccio = ' || central || '; Data_Inici = ' || TO_CHAR(data_inici)
  || '; Data_Fi = ' || TO_CHAR(data_fi) || ';';

 -- Excepci exc_dates_malament, que es dona si la data inici s posterior a la data fi
 IF data_inici > data_fi THEN
    RAISE exc_dates_malament;
 END IF;

 -- Excepci exc_no_contadors, que es dona si no hi ha lectures de cap contador
 -- per la central de producci i l'intrval de temps donats
 SELECT COUNT(*) INTO trobats FROM LECTURES
  WHERE data_lectura >= data_inici AND data_lectura <= data_fi AND
  codi_contracte IN (SELECT DISTINCT codi_contracte FROM CENTRALETES_DE_CONTADORS
  WHERE codi_central IN (SELECT DISTINCT codi_central_distribucio FROM LINIES_DE_COMUNICACIO
  WHERE codi_central_produccio = central));
 IF trobats = 0 THEN
    RAISE exc_no_contadors;
 END IF;

 OPEN refcursor FOR
  SELECT codi_contracte, SUM(ultima_lectura - primera_lectura) OVER(PARTITION BY codi_contracte),
   SUM(ultima_lectura - primera_lectura) OVER() FROM
   (SELECT DISTINCT codi_contracte, MAX(valor_lectura) OVER(PARTITION BY codi_contracte) as ultima_lectura,
    MIN(valor_lectura) OVER(PARTITION BY codi_contracte) as primera_lectura
    FROM LECTURES 
    WHERE data_lectura >= data_inici AND data_lectura <= data_fi AND
    codi_contracte IN (SELECT DISTINCT codi_contracte FROM CENTRALETES_DE_CONTADORS
    WHERE codi_central IN (SELECT DISTINCT codi_central_distribucio FROM LINIES_DE_COMUNICACIO
    WHERE codi_central_produccio = central)));
  
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

 -- mostra els resultats
 LOOP
    FETCH refcursor INTO linia;
    EXIT WHEN refcursor%NOTFOUND;
    DBMS_OUTPUT.PUT_LINE(linia.contador || ' - ' || TO_CHAR(linia.consum)
     || ' - ' || TO_CHAR(linia.total));
 END LOOP;
 CLOSE refcursor;
  
EXCEPTION
 WHEN exc_no_contadors THEN 
    resultat := 'ERROR. No hi ha lectures de cap contador per aquesta central de producci i aquest 
     intrval.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
 WHEN exc_dates_malament THEN 
    resultat := 'ERROR. L''intrval de temps indicat no s correcte.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
END;
/




/* Procediment que, donades una data d'inici i una data de fi,
   retorna els percentatges de cadascuna de les diferents modalitats de lectura utilitzades. */
create or replace PROCEDURE pr_modal_lectura (data_inici DATE, data_fi DATE,
 refcursor OUT SYS_REFCURSOR, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;

 -- Variables per excepcions 
 exc_no_lectures EXCEPTION;
 exc_dates_malament EXCEPTION;

 -- resultats
 linia sortidaLectures%rowtype;
    
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_modal_lectura';
 parametres_entrada := 'Data_Inici = ' || TO_CHAR(data_inici)
  || '; Data_Fi = ' || TO_CHAR(data_fi) || ';';

 -- Excepci exc_interval_malament, que es dona si la data inici s posterior a la data fi
 IF data_inici > data_fi THEN
    RAISE exc_dates_malament;
 END IF;

 -- Excepci exc_no_lectures, que es dona si no hi ha lectures de l'intrval de temps donat
 SELECT COUNT(*) INTO trobats FROM LECTURES
  WHERE data_lectura >= data_inici AND data_lectura <= data_fi;
 IF trobats = 0 THEN
    RAISE exc_no_lectures;
 END IF;

 OPEN refcursor FOR
  SELECT a.nom_modalitat, RATIO_TO_REPORT(a.suma_modal) OVER()
   FROM
    (SELECT m.nom_modalitat, COUNT(l.valor_lectura) AS suma_modal
     FROM MODALITATS_LECTURA m, LECTURES l
     WHERE l.data_lectura >= data_inici AND l.data_lectura <= data_fi
     AND m.codi_modalitat = l.modalitat_lectura
     GROUP BY m.nom_modalitat) a;
   
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

 -- mostra els resultats
 LOOP
    FETCH refcursor INTO linia;
    EXIT WHEN refcursor%NOTFOUND;
    DBMS_OUTPUT.PUT_LINE(linia.modalitat || ' - ' || TO_CHAR(linia.percentatge));
 END LOOP;
 CLOSE refcursor;
  
EXCEPTION
 WHEN exc_no_lectures THEN 
    resultat := 'ERROR. No hi ha lectures de cap contador per l''intrval de temps indicat.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
 WHEN exc_dates_malament THEN 
    resultat := 'ERROR. L''intrval de temps indicat no s correcte.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
END;
/





/* Procediment que, donat un nombre d'anys n,
   retorna els contadors amb una antiguitat igual o superior a aquest nombre d'anys n. */
create or replace PROCEDURE pr_contadors_antics (antiguitat INTEGER,
 refcursor OUT SYS_REFCURSOR, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;

 -- Variables per excepcions 
 exc_no_contadors EXCEPTION;

 -- resultats
 linia sortidaAntiguitat%rowtype;
    
BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'pr_contadors_antics';
 parametres_entrada := 'Antiguitat = ' || TO_CHAR(antiguitat) || ';';

 -- Excepci exc_no_lectures, que es dona si no hi ha contadors amb l'antiguitat donada
 SELECT COUNT(*) INTO trobats FROM CONTADORS
  WHERE any_fabricacio <= EXTRACT(YEAR FROM CURRENT_DATE)-antiguitat;
 IF trobats = 0 THEN
    RAISE exc_no_contadors;
 END IF;

 OPEN refcursor FOR
  SELECT codi_contracte, EXTRACT(YEAR FROM CURRENT_DATE)-any_fabricacio
   FROM CONTADORS WHERE any_fabricacio <= EXTRACT(YEAR FROM CURRENT_DATE)-antiguitat
   AND baixa = 0;
   
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
  
 -- Confirma
 COMMIT;

 -- mostra els resultats
 LOOP
    FETCH refcursor INTO linia;
    EXIT WHEN refcursor%NOTFOUND;
    DBMS_OUTPUT.PUT_LINE(linia.contador || ' - ' || TO_CHAR(linia.antiguitat));
 END LOOP;
 CLOSE refcursor;
  
EXCEPTION
 WHEN exc_no_contadors THEN 
    resultat := 'ERROR. No hi ha contadors que tinguin ' || TO_CHAR(antiguitat) || ' anys o ms.';    
    INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);  
    COMMIT;
END;
/