*	Autor: David Malet
	Data: 18-05-2012
	Projecte: Sistema de Control Energtic
	Revisi: 1
	Base de Dades: Oracle 10g
*/


/* Procediments per la generaci i el manteniment del Mdul Estadstic

/* Procediment que actualitza, cada cop que es fa l'alta d'una lectura de contador,
   la taula EST_CONSUM_PER_CENTRAL, per la Central o les Centrals de Producci que correspongui.
   Les dades utilitzades seran les contingudes en la taula CP_HISTORIC_PRODUCCIO.
   Aquest procediment ser cridat doncs pel procediment pr_alta_lectura_contador,
   just desprs d'haver cridat el procediment pr_act_energia_produida */
create or replace PROCEDURE act_est_consum_central (contador VARCHAR, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 TYPE registre IS RECORD(codi_central VARCHAR(50), consum_total NUMBER);
 TYPE registres IS TABLE OF registre;
 taula_temp registres;
 trobats INTEGER;

 -- Variables per excepcions 
 -- En principi aquest procediment sempre es cridar desprs d'una alta de lectura i, per tant,
 -- no s'hauria de donar mai que en la taula CP_HISTORIC_PRODUCCIO no hi hagus dades.

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'act_est_consum_central';
 parametres_entrada := 'Codi_Contracte = ' || contador || ';';

 -- Obtenci del sumatori de consums per les centrals de producci implicades (aquelles de les
 -- que "penja" el contador a travs de les centraletes de distribuci).
  SELECT codi_central, SUM(energia_absoluta) OVER (PARTITION BY codi_central)
   BULK COLLECT INTO taula_temp FROM CP_HISTORIC_PRODUCCIO
   WHERE codi_central IN (SELECT DISTINCT codi_central_produccio FROM LINIES_DE_COMUNICACIO
   WHERE codi_central_distribucio IN (SELECT codi_central FROM CENTRALETES_DE_CONTADORS
   WHERE codi_contracte = contador));

  FOR i IN taula_temp.FIRST .. taula_temp.LAST LOOP
 -- Si s el primer cop que es crida el procediment, caldr inserir les files corresponents a les CP's
 -- Si no, caldr actualitzar-les
     SELECT COUNT(*) INTO trobats FROM EST_CONSUM_PER_CENTRAL
      WHERE codi_central = taula_temp(i).codi_central;
     IF trobats = 0 THEN
        INSERT INTO EST_CONSUM_PER_CENTRAL
         VALUES(taula_temp(i).codi_central, taula_temp(i).consum_total);
     ELSE
        UPDATE EST_CONSUM_PER_CENTRAL SET consum_total = consum_total + taula_temp(i).consum_total
	 WHERE codi_central = taula_temp(i).codi_central;
     END IF;
  END LOOP;
  
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
 
 -- Confirma
 COMMIT;

END;
/





/* Procediment que actualitza, cada cop que es fa l'alta d'una lectura de contador,
   la taula EST_ENERGIA_MITJA_ANY_LINIA, per la Lnia o les Lnies de comunicaci que correspongui.
   Les dades utilitzades seran les contingudes en la taula LECTURES.
   Aquest procediment ser cridat doncs pel procediment pr_alta_lectura_contador,
   cop inserida la lectura del contador. */
create or replace PROCEDURE act_est_energia_linia (contador VARCHAR, data_lect DATE,
 resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 TYPE registre IS RECORD(codi_linia VARCHAR(50), consum_mig NUMBER);
 TYPE registres IS TABLE OF registre;
 taula_temp registres;
 trobats INTEGER;
 inicial NUMBER;
 
 -- Variables per excepcions 
 -- En principi aquest procediment sempre es cridar desprs d'una alta de lectura i, per tant,
 -- no s'hauria de donar mai que en la taula LECTURES no hi hagus dades.

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'act_est_energia_linia';
 parametres_entrada := 'Codi_Contracte = ' || contador || ';';

 -- Obtenci de la mitja de consums per les lnies de comunicaci implicades (aquelles de les
 -- que "penja" el contador a travs de les centraletes de distribuci).
 -- Es crida a la funci consum_contador per calcular el consum de cada contador en l'any d'inters.

  SELECT linia, AVG(consum_contadors) OVER (PARTITION BY linia)
     BULK COLLECT INTO taula_temp FROM
     (SELECT DISTINCT lc.codi_linia AS linia, SUM(fn_consum_contador(cc.codi_contracte, data_lect))
     OVER (PARTITION BY lc.codi_linia) AS consum_contadors
     FROM LINIES_DE_COMUNICACIO lc, CENTRALETES_DE_CONTADORS cc
     WHERE cc.codi_central = lc.codi_central_distribucio
     AND cc.codi_contracte = contador);

  FOR i IN taula_temp.FIRST .. taula_temp.LAST LOOP
 -- Si s el primer cop que es crida el procediment, caldr inserir les files corresponents a les Lnies
 -- Si no, caldr actualitzar-les
     SELECT COUNT(*) INTO trobats FROM EST_ENERGIA_MITJA_ANY_LINIA
      WHERE codi_linia = taula_temp(i).codi_linia AND est_any = EXTRACT (YEAR FROM data_lect);
     IF trobats = 0 THEN
        INSERT INTO EST_ENERGIA_MITJA_ANY_LINIA
         VALUES(taula_temp(i).codi_linia, EXTRACT (YEAR FROM data_lect), taula_temp(i).consum_mig);
     ELSE
        UPDATE EST_ENERGIA_MITJA_ANY_LINIA SET consum_mig = taula_temp(i).consum_mig
	 WHERE codi_linia = taula_temp(i).codi_linia AND est_any = EXTRACT (YEAR FROM data_lect);
     END IF;
  END LOOP;
  
  resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
 
 -- Confirma
 COMMIT;

END;
/





/* Procediment que actualitza, cada cop que es fa l'alta d'una lectura de contador,
   la taula EST_LINIA_MES_CARREGADA, que cont una sola fila amb la lnia de comunicaci
   ms carregada, segons les dades de la taula CD_HISTORIC_EMISSIO. */
create or replace PROCEDURE act_est_linia_mes_carregada (resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;
 linia VARCHAR(50);
 carrega NUMBER;
 TYPE registre IS RECORD(codi_linia VARCHAR(50));
 TYPE registres IS TABLE OF registre;
 taula_temp registres;


 -- Variables per excepcions 
 -- En principi aquest procediment sempre es cridar desprs d'una alta de lectura i, per tant,
 -- no s'hauria de donar mai que en la taula CD_HISTORIC_EMISSIO no hi hagus dades.

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'act_est_linia_mes_carregada';
 parametres_entrada := '';

 -- Obt el valor mxim de consum de les lnies, que es correspon amb el mxim valor
 -- d'emissi de les centraletes de distribuci associades, considerant que algunes reben
 -- energia de ms d'una lnia i la capacitat mxima de les lnies.
 -- L'emissi s en KWh mensuals, i la capacitat de la lnia en KW. Aix, considerarem
 -- un mes de 30 dies, s a dir, 720 hores.

 SELECT carrega_maxima INTO carrega FROM
  (SELECT MAX(h.energia_absoluta*fn_frac_pot_centraleta(h.codi_central)/(li.capacitat_maxima*720))
  OVER () AS carrega_maxima FROM LINIES_DE_COMUNICACIO li, CD_HISTORIC_EMISSIO h
  WHERE li.codi_central_distribucio = h.codi_central)
  WHERE ROWNUM = 1;

 -- Podria haver-hi ms d'una lnia igual de carregada, aix que utilitzem una taula temporal
 SELECT codi_linia BULK COLLECT INTO taula_temp
  FROM LINIES_DE_COMUNICACIO li, CD_HISTORIC_EMISSIO h
  WHERE li.codi_central_distribucio = h.codi_central
  AND h.energia_absoluta*fn_frac_pot_centraleta(h.codi_central)/(li.capacitat_maxima*720) = carrega;

  DELETE FROM EST_LINIA_MES_CARREGADA;

  FOR i IN taula_temp.FIRST .. taula_temp.LAST LOOP
     INSERT INTO EST_LINIA_MES_CARREGADA
      VALUES(taula_temp(i).codi_linia, 100*carrega);
  END LOOP;

 resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
 
 -- Confirma
 COMMIT;

END;
/




/* Procediment que actualitza, cada cop que es fa l'alta d'una lectura de contador,
   la taula EST_PERC_LINIES_MES_50PCNT, que cont, per cada any,
   el percentatge del total de linies que suporten ms del 50%
   d'energia consumida, repecte de la seva capacitat mxima,
   segons les dades de la taula CD_HISTORIC_EMISSIO.
   Com que les dades de CD_HISTORIC_EMISSIO son en KWh mensuals i la capacitat de la linia
   s en KW, considerarem un mes de 30 dies, s a dir, 720 hores. */
create or replace PROCEDURE act_est_linies_mes_50pcnt (data_interes DATE, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;
 total_linies INTEGER;
 nombre_linies INTEGER;

 -- Variables per excepcions 
 -- En principi aquest procediment sempre es cridar desprs d'una alta de lectura i, per tant,
 -- no s'hauria de donar mai que en la taula CD_HISTORIC_EMISSIO no hi hagus dades.

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'act_est_linies_mes_50pcnt';
 parametres_entrada := 'Data_inters = ' || data_interes || ';';

 -- Necessito el nombre total de lnies
 SELECT COUNT(*) INTO total_linies FROM LINIES_DE_COMUNICACIO;

 SELECT COUNT(linia_com) INTO nombre_linies FROM
  (SELECT l.codi_linia AS linia_com,
   AVG(h.energia_absoluta * fn_frac_pot_centraleta(h.codi_central))
   OVER (PARTITION BY h.codi_central) AS consum_mensual 
   FROM LINIES_DE_COMUNICACIO l, CD_HISTORIC_EMISSIO h
   WHERE h.codi_central = l.codi_central_distribucio
   AND EXTRACT (YEAR FROM h.data_historic) = EXTRACT (YEAR FROM data_interes))
   WHERE consum_mensual/720 > 0.5*(SELECT capacitat_maxima FROM LINIES_DE_COMUNICACIO
   WHERE codi_linia = linia_com);

 -- Si no hi ha cap fila a EST_PERC_LINIES_MES_50PCNT per l'any d'inters l'haur d'inserir de nou
 -- Si hi s, la modificar amb els valors calculats
 SELECT COUNT(*) INTO trobats FROM EST_PERC_LINIES_MES_50PCNT
  WHERE est_any = EXTRACT (YEAR FROM data_interes);
 IF trobats = 0 THEN
    INSERT INTO EST_PERC_LINIES_MES_50PCNT
     VALUES(EXTRACT (YEAR FROM data_interes), 100*nombre_linies/total_linies);
 ELSE
    UPDATE EST_PERC_LINIES_MES_50PCNT SET percentatge_linies = 100*nombre_linies/total_linies
     WHERE est_any = EXTRACT (YEAR FROM data_interes);
 END IF;
  
 resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
 
 -- Confirma
 COMMIT;
END;
/





/* Procediment que actualitza, cada cop que es fa l'alta d'una lectura de contador,
   la taula EST_N_CENTRALS_MENYS_30PCNT, que cont, per cada any,
   el nombre de centrals de producci que generen menys del 30% d'energia
   repecte de la seva capacitat mensual mxima,
   segons les dades de la taula CP_HISTORIC_PRODUCCIO. */
create or replace PROCEDURE act_est_centrals_menys_30pcnt (data_interes DATE, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;
 num_centrals INTEGER;
 sql_string VARCHAR(500);

 -- Variables per excepcions 
 -- En principi aquest procediment sempre es cridar desprs d'una alta de lectura i, per tant,
 -- no s'hauria de donar mai que en la taula CD_HISTORIC_EMISSIO no hi hagus dades.

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'act_est_centrals_menys_30pcnt';
 parametres_entrada := 'Data_inters = ' || data_interes || ';';

 SELECT COUNT(central) INTO num_centrals FROM
  (SELECT central, consum_mensual FROM
   (SELECT DISTINCT codi_central AS central,
    AVG(energia_absoluta) OVER (PARTITION BY codi_central) consum_mensual
    FROM CP_HISTORIC_PRODUCCIO
    WHERE EXTRACT (YEAR FROM data_historic) = EXTRACT (YEAR FROM data_interes))
   WHERE consum_mensual < 0.3*fn_capacitat_CP(central));

 -- Si no hi ha cap fila a EST_N_CENTRALS_MENYS_30PCNT per l'any d'inters l'haur d'inserir de nou
 -- Si hi s, la modificar amb els valors calculats
 SELECT COUNT(*) INTO trobats FROM EST_N_CENTRALS_MENYS_30PCNT
  WHERE est_any = EXTRACT (YEAR FROM data_interes);
 IF trobats = 0 THEN
    INSERT INTO EST_N_CENTRALS_MENYS_30PCNT
     VALUES(EXTRACT (YEAR FROM data_interes), num_centrals);
 ELSE
    UPDATE EST_N_CENTRALS_MENYS_30PCNT SET nombre_centrals = num_centrals
     WHERE est_any = EXTRACT (YEAR FROM data_interes);
 END IF;
  
 resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
 
 -- Confirma
 COMMIT;

END;
/




/* Procediment que actualitza, cada cop que es fa l'alta d'una lectura de contador,
   la taula EST_10_CONTADORS_MAXIM_CONSUM, que cont els 10 contadors amb major consum histric,
   segons les dades de la taula LECTURES. */
create or replace PROCEDURE act_est_top10_contadors (resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;
 TYPE registre IS RECORD(codi_contracte VARCHAR(50), consum_total NUMBER);
 TYPE registres IS TABLE OF registre;
 taula_temp registres;

 -- Variables per excepcions 
 -- En principi aquest procediment sempre es cridar desprs d'una alta de lectura i, per tant,
 -- no s'hauria de donar mai que en la taula LECTURES no hi hagus dades.

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'act_est_top10_contadors';
 parametres_entrada := '';

 SELECT COUNT(*) INTO trobats FROM CONTADORS;

 SELECT contador, consum_acumulat BULK COLLECT INTO taula_temp FROM
  (SELECT DISTINCT codi_contracte AS contador,
  MAX(valor_lectura) OVER (PARTITION BY codi_contracte) AS consum_acumulat
  FROM LECTURES ORDER BY consum_acumulat DESC)
  WHERE ROWNUM < 11 AND ROWNUM < trobats;

 DELETE FROM EST_10_CONTADORS_MAXIM_CONSUM;
 
 FOR i IN taula_temp.FIRST .. taula_temp.LAST LOOP
    INSERT INTO EST_10_CONTADORS_MAXIM_CONSUM
     VALUES(taula_temp(i).codi_contracte, taula_temp(i).consum_total);
 END LOOP;
  
 resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
 
 -- Confirma
 COMMIT;

END;
/




/* Procediment que actualitza, cada cop que es fa l'alta d'una lectura de contador,
   la taula EST_CONSUM_MIG_PER_CLIENT, que cont el consum mig de cada client,
   s a dir, la suma dels consums mitjos dels contadors del client,
   segons les dades de la taula CONTADORS (que cont el consum mig del contador). */
create or replace PROCEDURE act_est_consum_clients (contador VARCHAR, resultat OUT CLOB) IS

 -- Variables per actualitzar la taula LOGS
 data_hora VARCHAR(20);
 nom_procediment VARCHAR(50);
 parametres_entrada CLOB;

 -- Variables de consulta
 trobats INTEGER;
 nif_client VARCHAR(15);
 consum NUMBER;

 -- Variables per excepcions 
 -- En principi aquest procediment sempre es cridar desprs d'una alta de lectura i, per tant,
 -- no s'hauria de donar mai que en la taula CONTADORS no hi hagus les dades que necessitem.

BEGIN
 -- Inicialitza variables de la taula de log
 data_hora := TO_CHAR(CURRENT_DATE, 'YYYY-MM-DD HH24:MI:SS');
 nom_procediment := 'act_est_consum_clients';
 parametres_entrada := 'Codi_Contracte = ' || contador || ';';

 SELECT client, SUM(consum_mensual_mig) OVER (PARTITION BY client)
  INTO nif_client, consum FROM CONTADORS
  WHERE codi_contracte = contador;

 -- Si no hi ha fila a EST_CONSUM_MIG_PER_CLIENT pel client en qesti, l'haur d'inserir
 -- Si hi s, la modificar amb els valors calculats
 SELECT COUNT(*) INTO trobats FROM EST_CONSUM_MIG_PER_CLIENT WHERE NIF = nif_client;
 IF trobats = 0 THEN
    INSERT INTO EST_CONSUM_MIG_PER_CLIENT VALUES(nif_client, consum);
 ELSE
    UPDATE EST_CONSUM_MIG_PER_CLIENT SET consum_mig_total = consum
     WHERE NIF = nif_client;
 END IF;
  
 resultat := 'OK';
    
 -- Inserci de la informaci sobre la crida al procediment a la taula LOGS
 INSERT INTO LOGS VALUES (data_hora, nom_procediment, parametres_entrada, resultat);
 
 -- Confirma
 COMMIT;

END;
/