/*	Autor: David Malet
	Data: 13-04-2012
	Projecte: Sistema de Control Energtic
	Revisi: 3
	Base de Dades: Oracle 10g
*/

/* Creaci de taules i inicialitzaci si s'escau */

-- Neteja inicial de taules
DROP TABLE LOGS CASCADE CONSTRAINTS;
DROP TABLE CD_HISTORIC_EMISSIO CASCADE CONSTRAINTS;
DROP TABLE CP_HISTORIC_PRODUCCIO CASCADE CONSTRAINTS;
DROP TABLE LECTURES CASCADE CONSTRAINTS;
DROP TABLE MODALITATS_LECTURA CASCADE CONSTRAINTS;
DROP TABLE CENTRALETES_DE_CONTADORS CASCADE CONSTRAINTS;
DROP TABLE LINIES_DE_COMUNICACIO CASCADE CONSTRAINTS;
DROP TABLE CENTRALS_DE_DISTRIBUCIO CASCADE CONSTRAINTS;
DROP TABLE CENTRALS_NUCLEARS CASCADE CONSTRAINTS;
DROP TABLE CENTRALS_TERMIQUES CASCADE CONSTRAINTS;
DROP TABLE CENTRALS_DE_CARBO CASCADE CONSTRAINTS;
DROP TABLE CENTRALS_EOLIQUES CASCADE CONSTRAINTS;
DROP TABLE CENTRALS_SOLARS CASCADE CONSTRAINTS;
DROP TABLE TAULES_DE_CENTRALS CASCADE CONSTRAINTS;
DROP TABLE CONTADORS CASCADE CONSTRAINTS;
DROP TABLE PERSONES CASCADE CONSTRAINTS;

-- Taula amb les dades de les PERSONES (en principi, seran els Clients)
CREATE TABLE PERSONES (
	NIF VARCHAR(15) NOT NULL PRIMARY KEY,
	nom VARCHAR(30) NOT NULL,
	cognoms VARCHAR(75) NOT NULL,
	adrea VARCHAR(150) NOT NULL,
	codi_postal NUMBER(5) NOT NULL,
	poblacio VARCHAR(75) NOT NULL,
	provincia VARCHAR(40) NOT NULL,
	pais VARCHAR(25) NOT NULL
);

-- Taula amb les dades dels CONTADORS
CREATE TABLE CONTADORS (
	codi_contracte VARCHAR(50) NOT NULL PRIMARY KEY,
	potencia_contractada NUMBER(*,3) NOT NULL,
	adrea VARCHAR(150) NOT NULL,
	codi_postal NUMBER(5) NOT NULL,
	poblacio VARCHAR(75) NOT NULL,
	provincia VARCHAR(40) NOT NULL,
	pais VARCHAR(25) NOT NULL,
	consum_mensual_mig NUMBER(*,2),
	data_darrera_lectura DATE,
	marca VARCHAR(25),
	model_c VARCHAR(25),
	any_fabricacio NUMBER(4),
	numero_serie VARCHAR(50),
	servei_alta_disponibilitat NUMBER(1) NOT NULL,
	client VARCHAR(15) NOT NULL,
	baixa NUMBER(1) NOT NULL,
	CONSTRAINT FK_CLIENT FOREIGN KEY (client) REFERENCES PERSONES(NIF)
);

-- Taula amb les relacions entre cada central de producci i la taula amb les seves dades,
-- segons el tipus de central (distribuci, nuclear, elica...)
CREATE TABLE TAULES_DE_CENTRALS (
	codi_central VARCHAR(50) NOT NULL PRIMARY KEY,	
	nom_taula VARCHAR(25) NOT NULL
);

-- Taula amb les dades de les CENTRALS NUCLEARS
CREATE TABLE CENTRALS_NUCLEARS (
	codi_central VARCHAR(50) NOT NULL PRIMARY KEY,
	adrea VARCHAR(150) NOT NULL,
	codi_postal NUMBER(5) NOT NULL,
	poblacio VARCHAR(75) NOT NULL,
	provincia VARCHAR(40) NOT NULL,
	pais VARCHAR(25) NOT NULL,
	data_darrera_inspeccio DATE,
	capacitat_maxima NUMBER(*,2) NOT NULL,
	energia_minima_de_funcionament NUMBER(*,2) NOT NULL,
	CONSTRAINT FK_CP_NUCLEAR FOREIGN KEY (codi_central)
	 REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les dades de les CENTRALS TRMIQUES
CREATE TABLE CENTRALS_TERMIQUES (
	codi_central VARCHAR(50) NOT NULL PRIMARY KEY,
	adrea VARCHAR(150) NOT NULL,
	codi_postal NUMBER(5) NOT NULL,
	poblacio VARCHAR(75) NOT NULL,
	provincia VARCHAR(40) NOT NULL,
	pais VARCHAR(25) NOT NULL,
	data_darrera_inspeccio DATE,
	capacitat_maxima NUMBER(*,2) NOT NULL,
	emissions_CO2 NUMBER(*,2),
	CONSTRAINT FK_CP_TERMICA FOREIGN KEY (codi_central)
	 REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les dades de les CENTRALS DE CARB
CREATE TABLE CENTRALS_DE_CARBO (
	codi_central VARCHAR(50) NOT NULL PRIMARY KEY,
	adrea VARCHAR(150) NOT NULL,
	codi_postal NUMBER(5) NOT NULL,
	poblacio VARCHAR(75) NOT NULL,
	provincia VARCHAR(40) NOT NULL,
	pais VARCHAR(25) NOT NULL,
	data_darrera_inspeccio DATE,
	capacitat_maxima NUMBER(*,2) NOT NULL,
	emissions_CO2 NUMBER(*,2),
	CONSTRAINT FK_CP_CARBO FOREIGN KEY (codi_central)
	 REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les dades de les CENTRALS ELIQUES
CREATE TABLE CENTRALS_EOLIQUES (
	codi_central VARCHAR(50) NOT NULL PRIMARY KEY,
	adrea VARCHAR(150) NOT NULL,
	codi_postal NUMBER(5) NOT NULL,
	poblacio VARCHAR(75) NOT NULL,
	provincia VARCHAR(40) NOT NULL,
	pais VARCHAR(25) NOT NULL,
	data_darrera_inspeccio DATE,
	capacitat_maxima NUMBER(*,2) NOT NULL,
	numero_de_molins INTEGER,
	CONSTRAINT FK_CP_EOLICA FOREIGN KEY (codi_central)
	 REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les dades de les CENTRALS SOLARS
CREATE TABLE CENTRALS_SOLARS (
	codi_central VARCHAR(50) NOT NULL PRIMARY KEY,
	adrea VARCHAR(150) NOT NULL,
	codi_postal NUMBER(5) NOT NULL,
	poblacio VARCHAR(75) NOT NULL,
	provincia VARCHAR(40) NOT NULL,
	pais VARCHAR(25) NOT NULL,
	data_darrera_inspeccio DATE,
	capacitat_maxima NUMBER(*,2) NOT NULL,
	numero_panells_funcionament INTEGER NOT NULL,
	CONSTRAINT FK_CP_SOLAR FOREIGN KEY (codi_central)
	 REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les dades de les CENTRALS DE DISTRIBUCI
CREATE TABLE CENTRALS_DE_DISTRIBUCIO (
	codi_central VARCHAR(50) NOT NULL PRIMARY KEY,
	adrea VARCHAR(150) NOT NULL,
	codi_postal NUMBER(5) NOT NULL,
	poblacio VARCHAR(75) NOT NULL,
	provincia VARCHAR(40) NOT NULL,
	pais VARCHAR(25) NOT NULL,
	data_darrera_inspeccio DATE,
	capacitat_maxima NUMBER(*,2) NOT NULL,
	CONSTRAINT FK_DISTRIBUCIO FOREIGN KEY (codi_central)
	 REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les dades de les LINIES DE COMUNICACI
CREATE TABLE LINIES_DE_COMUNICACIO (
	codi_linia VARCHAR(50) NOT NULL PRIMARY KEY,
	codi_central_produccio VARCHAR(50) NOT NULL,
	codi_central_distribucio VARCHAR(50) NOT NULL,
	capacitat_maxima NUMBER(*,2) NOT NULL,
	CONSTRAINT FK_CENTRAL_PRODUCCIO FOREIGN KEY (codi_central_produccio)
	REFERENCES TAULES_DE_CENTRALS(codi_central),
	CONSTRAINT FK_CENTRAL_DISTRIBUCIO FOREIGN KEY (codi_central_distribucio)
	REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les relacions entre cada CONTADOR i les CENTRALETES DE DISTRIBUCI
-- de les quals reb subministre elctric
CREATE TABLE CENTRALETES_DE_CONTADORS (
	codi_contracte VARCHAR(50) NOT NULL,
	codi_central VARCHAR(50) NOT NULL,
	CONSTRAINT PK_CENTRALETES_CONTADORS PRIMARY KEY (codi_contracte,codi_central),
	CONSTRAINT FK_CONTRACTE FOREIGN KEY (codi_contracte)
	REFERENCES CONTADORS(codi_contracte),
	CONSTRAINT FK_CENTRAL FOREIGN KEY (codi_central)
	REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les MODALITATS DE LECTURA de contador
CREATE TABLE MODALITATS_LECTURA (
	codi_modalitat INTEGER NOT NULL PRIMARY KEY,
	nom_modalitat VARCHAR(15) NOT NULL
);

-- Taula amb les dades de les LECTURES
CREATE TABLE LECTURES (
	codi_contracte VARCHAR(50) NOT NULL,
	data_lectura DATE NOT NULL,
	valor_lectura NUMBER(*,2) NOT NULL,
	modalitat_lectura INTEGER,
	CONSTRAINT PK_LECTURES PRIMARY KEY (codi_contracte,data_lectura),
	CONSTRAINT FK_CONTRACTE_LECTURA FOREIGN KEY (codi_contracte)
	REFERENCES CONTADORS(codi_contracte),
	CONSTRAINT FK_MODALITAT_LECTURA FOREIGN KEY (modalitat_lectura)
	REFERENCES MODALITATS_LECTURA(codi_modalitat)
);

-- Taula amb les dades histriques de producci mensual de les centrals de producci
CREATE TABLE CP_HISTORIC_PRODUCCIO (
	codi_central VARCHAR(50) NOT NULL,
	data_historic DATE NOT NULL,
	energia_absoluta NUMBER(*,2),
	CONSTRAINT PK_HISTORIC_PRODUCCIO PRIMARY KEY (codi_central,data_historic),
	CONSTRAINT FK_HISTORIC_PRODUCCIO FOREIGN KEY (codi_central)
	 REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les dades histriques d'emissi mensual de les centraletes de distribuci
CREATE TABLE CD_HISTORIC_EMISSIO (
	codi_central VARCHAR(50) NOT NULL,
	data_historic DATE NOT NULL,
	energia_absoluta NUMBER(*,2),
	CONSTRAINT PK_HISTORIC_EMISSIO PRIMARY KEY (codi_central,data_historic),
	CONSTRAINT FK_HISTORIC_EMISSIO FOREIGN KEY (codi_central)
	 REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb el registre d'activitat dels procediments emmagatzemats
CREATE TABLE LOGS (
	data_hora VARCHAR(20) NOT NULL,
	nom_procediment VARCHAR(50) NOT NULL,
	parametres_entrada CLOB,
	parametrs_sortida CLOB
);

-- Dades d'inicialitzaci necessries
INSERT INTO MODALITATS_LECTURA VALUES (1,'TELEMTICA');
INSERT INTO MODALITATS_LECTURA VALUES (2,'PRESENCIAL');
INSERT INTO MODALITATS_LECTURA VALUES (3,'ESTIMADA');
