/*	Autor: David Malet
	Data: 11-05-2012
	Projecte: Sistema de Control Energtic
	Revisi: 2
	Base de Dades: Oracle 10g
*/

/* Mdul Estadstic */
/* Creaci de taules i inicialitzaci si s'escau */

-- Neteja inicial de taules
DROP TABLE EST_CONSUM_PER_CENTRAL CASCADE CONSTRAINTS;
DROP TABLE EST_ENERGIA_MITJA_ANY_LINIA CASCADE CONSTRAINTS;
DROP TABLE EST_LINIA_MES_CARREGADA CASCADE CONSTRAINTS;
DROP TABLE EST_PERC_LINIES_MES_50PCNT CASCADE CONSTRAINTS;
DROP TABLE EST_N_CENTRALS_MENYS_30PCNT CASCADE CONSTRAINTS;
DROP TABLE EST_10_CONTADORS_MAXIM_CONSUM CASCADE CONSTRAINTS;
DROP TABLE EST_CONSUM_MIG_PER_CLIENT CASCADE CONSTRAINTS;

-- Taula amb les dades sobre els consums totals de les centrals de producci
CREATE TABLE EST_CONSUM_PER_CENTRAL (
	codi_central VARCHAR(50) NOT NULL PRIMARY KEY,
	consum_total NUMBER(*,2),
	CONSTRAINT FK_EST_CENTRAL FOREIGN KEY (codi_central) REFERENCES TAULES_DE_CENTRALS(codi_central)
);

-- Taula amb les dades sobre els consums mitjos, per anys, de les lnies de comunicaci
CREATE TABLE EST_ENERGIA_MITJA_ANY_LINIA (
	codi_linia VARCHAR(50) NOT NULL,
	est_any NUMBER(4) NOT NULL,
	consum_mig NUMBER(*,2),
	CONSTRAINT PK_EST_ENERGIA PRIMARY KEY (codi_linia, est_any),
	CONSTRAINT FK_EST_LINIA FOREIGN KEY (codi_linia) REFERENCES LINIES_DE_COMUNICACIO(codi_linia)
);

CREATE TABLE EST_LINIA_MES_CARREGADA (
	codi_linia VARCHAR(50) NOT NULL PRIMARY KEY,
	energia_maxima_consumida NUMBER(*,2),
	CONSTRAINT FK_EST_LINIA2 FOREIGN KEY (codi_linia) REFERENCES LINIES_DE_COMUNICACIO(codi_linia)
);

-- Taula amb les dades sobre el percentatge de lnies de comunicaci, per anys,
-- que suporten ms del 50% de l'energia consumida
CREATE TABLE EST_PERC_LINIES_MES_50PCNT (
	est_any NUMBER(4) NOT NULL PRIMARY KEY,
	percentatge_linies NUMBER(*,2)
);

-- Taula amb les dades sobre el nombre de centrals de producci, per anys,
-- que generen menys del 30% de l'energia total produda
CREATE TABLE EST_N_CENTRALS_MENYS_30PCNT (
	est_any NUMBER(4) NOT NULL PRIMARY KEY,
	nombre_centrals INTEGER
);

-- Taula amb les dades sobre els 10 contadors que de manera acumulada,
-- han tingut el consum ms elevat
CREATE TABLE EST_10_CONTADORS_MAXIM_CONSUM (
	codi_contracte VARCHAR(50) NOT NULL PRIMARY KEY,
	consum_acumulat NUMBER(*,2),
	CONSTRAINT FK_EST_CONTADOR FOREIGN KEY (codi_contracte) REFERENCES CONTADORS(codi_contracte)
);

-- Taula amb les dades sobre el consum mig dels contadors de cadascun dels clients
CREATE TABLE EST_CONSUM_MIG_PER_CLIENT (
	NIF VARCHAR(15) NOT NULL PRIMARY KEY,
	consum_mig_total NUMBER(*,2),
	CONSTRAINT FK_EST_CLIENT FOREIGN KEY (NIF) REFERENCES PERSONES(NIF)
);